"""
Test lldb-dap threads request
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import lldbdap_testcase


class TestDAP_threads(lldbdap_testcase.DAPTestCaseBase):
    def test_correct_thread(self):
        """
        Tests that the correct thread is selected if we continue from
        a thread that goes away and hit a breakpoint in another thread.
        In this case, the selected thread should be the thread that
        just hit the breakpoint, and not the first thread in the list.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        source = "main.cpp"
        breakpoint_line = line_number(source, "// break here")
        lines = [breakpoint_line]
        # Set breakpoint in the thread function
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(
            len(breakpoint_ids), len(lines), "expect correct number of breakpoints"
        )
        self.continue_to_breakpoints(breakpoint_ids)
        # We're now stopped at the breakpoint in the first thread, thread #2.
        # Continue to join the first thread and hit the breakpoint in the
        # second thread, thread #3.
        self.dap_server.request_continue()
        stopped_event = self.dap_server.wait_for_stopped()
        # Verify that the description is the relevant breakpoint,
        # preserveFocusHint is False and threadCausedFocus is True
        self.assertTrue(
            stopped_event[0]["body"]["description"].startswith(
                "breakpoint %s." % breakpoint_ids[0]
            )
        )
        self.assertNotIn("preserveFocusHint", stopped_event[0]["body"])
        # All threads should be named Thread {index}
        threads = self.dap_server.get_threads()
        self.assertTrue(all(len(t["name"]) > 0 for t in threads))

    def test_thread_format(self):
        """
        Tests the support for custom thread formats.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(
            program,
            customThreadFormat="This is thread index #${thread.index}",
            stopCommands=["thread list"],
        )
        source = "main.cpp"
        breakpoint_line = line_number(source, "// break here")
        lines = [breakpoint_line]
        # Set breakpoint in the thread function
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(
            len(breakpoint_ids), len(lines), "expect correct number of breakpoints"
        )
        self.continue_to_breakpoints(breakpoint_ids)
        # We are stopped at the first thread
        threads = self.dap_server.get_threads()
        print("got thread", threads)
        if self.getPlatform() == "windows":
            # Windows creates a thread pool once WaitForSingleObject is called
            # by thread.join(). As we are in the thread function, we can't be
            # certain that join() has been called yet and a thread pool has
            # been created, thus we only check for the first two threads.
            names = list(sorted(t["name"] for t in threads))[:2]
            self.assertEqual(
                names, ["This is thread index #1", "This is thread index #2"]
            )
        else:
            self.assertEqual(threads[0]["name"], "This is thread index #1")
            self.assertEqual(threads[1]["name"], "This is thread index #2")



# RUN: %PYTHON %s | FileCheck %s

import gc
from tempfile import NamedTemporaryFile
from mlir.ir import *


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0
    return f


# Verify successful parse.
# CHECK-LABEL: TEST: testParseSuccess
# CHECK: module @successfulParse
@run
def testParseSuccess():
    ctx = Context()
    module = Module.parse(r"""module @successfulParse {}""", ctx)
    assert module.context is ctx
    print("CLEAR CONTEXT")
    ctx = None  # Ensure that module captures the context.
    gc.collect()
    module.dump()  # Just outputs to stderr. Verifies that it functions.
    print(str(module))


# Verify successful parse from file.
# CHECK-LABEL: TEST: testParseFromFileSuccess
# CHECK: module @successfulParse
@run
def testParseFromFileSuccess():
    ctx = Context()
    with NamedTemporaryFile(mode="w") as tmp_file:
        tmp_file.write(r"""module @successfulParse {}""")
        tmp_file.flush()
        module = Module.parseFile(tmp_file.name, ctx)
        assert module.context is ctx
        print("CLEAR CONTEXT")
        ctx = None  # Ensure that module captures the context.
        gc.collect()
        module.operation.verify()
        print(str(module))


# Verify parse error.
# CHECK-LABEL: TEST: testParseError
# CHECK: testParseError: <
# CHECK:   Unable to parse module assembly:
# CHECK:   error: "-":1:1: expected operation name in quotes
# CHECK: >
@run
def testParseError():
    ctx = Context()
    try:
        module = Module.parse(r"""}SYNTAX ERROR{""", ctx)
    except MLIRError as e:
        print(f"testParseError: <{e}>")
    else:
        print("Exception not produced")


# Verify successful parse.
# CHECK-LABEL: TEST: testCreateEmpty
# CHECK: module {
@run
def testCreateEmpty():
    ctx = Context()
    loc = Location.unknown(ctx)
    module = Module.create(loc)
    print("CLEAR CONTEXT")
    ctx = None  # Ensure that module captures the context.
    gc.collect()
    print(str(module))


# Verify round-trip of ASM that contains unicode.
# Note that this does not test that the print path converts unicode properly
# because MLIR asm always normalizes it to the hex encoding.
# CHECK-LABEL: TEST: testRoundtripUnicode
# CHECK: func private @roundtripUnicode()
# CHECK: foo = "\F0\9F\98\8A"
@run
def testRoundtripUnicode():
    ctx = Context()
    module = Module.parse(
        r"""
    func.func private @roundtripUnicode() attributes { foo = "😊" }
  """,
        ctx,
    )
    print(str(module))


# Verify round-trip of ASM that contains unicode.
# Note that this does not test that the print path converts unicode properly
# because MLIR asm always normalizes it to the hex encoding.
# CHECK-LABEL: TEST: testRoundtripBinary
# CHECK: func private @roundtripUnicode()
# CHECK: foo = "\F0\9F\98\8A"
@run
def testRoundtripBinary():
    with Context():
        module = Module.parse(
            r"""
      func.func private @roundtripUnicode() attributes { foo = "😊" }
    """
        )
        binary_asm = module.operation.get_asm(binary=True)
        assert isinstance(binary_asm, bytes)
        module = Module.parse(binary_asm)
        print(module)


# Tests that module.operation works and correctly interns instances.
# CHECK-LABEL: TEST: testModuleOperation
@run
def testModuleOperation():
    ctx = Context()
    module = Module.parse(r"""module @successfulParse {}""", ctx)
    assert ctx._get_live_module_count() == 1
    op1 = module.operation
    # CHECK: module @successfulParse
    print(op1)

    # Ensure that operations are the same on multiple calls.
    op2 = module.operation
    assert op1 is not op2
    assert op1 == op2

    # Test live operation clearing.
    op1 = module.operation
    op1 = None
    gc.collect()
    op1 = module.operation

    # Ensure that if module is de-referenced, the operations are still valid.
    module = None
    gc.collect()
    print(op1)

    # Collect and verify lifetime.
    op1 = None
    op2 = None
    gc.collect()
    assert ctx._get_live_module_count() == 0


# CHECK-LABEL: TEST: testModuleCapsule
@run
def testModuleCapsule():
    ctx = Context()
    module = Module.parse(r"""module @successfulParse {}""", ctx)
    assert ctx._get_live_module_count() == 1
    # CHECK: "mlir.ir.Module._CAPIPtr"
    module_capsule = module._CAPIPtr
    print(module_capsule)
    module_dup = Module._CAPICreate(module_capsule)
    assert module is module_dup
    assert module == module_dup
    assert module_dup.context is ctx
    # Gc and verify destructed.
    module = None
    module_capsule = None
    module_dup = None
    gc.collect()
    assert ctx._get_live_module_count() == 0



from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @skipIf  # rdar://problem/53931074
    def test(self):
        self.build()
        target = self.createTestTarget()
        callee_break = target.BreakpointCreateByName(
            "SomeClass::SomeClass(ParamClass)", None
        )
        self.assertGreater(callee_break.GetNumLocations(), 0)
        self.runCmd("run", RUN_SUCCEEDED)

        to_complete = "e ParamClass"
        self.dbg.GetCommandInterpreter().HandleCompletion(
            to_complete, len(to_complete), 0, -1, lldb.SBStringList()
        )



"""
Test expression command options.

Test cases:

o test_expr_options:
  Test expression command options.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class ExprOptionsTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        self.main_source = "main.cpp"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)
        self.line = line_number("main.cpp", "// breakpoint_in_main")
        self.exe = self.getBuildArtifact("a.out")

    def test_expr_options(self):
        """These expression command options should work as expected."""
        self.build()

        # Set debugger into synchronous mode
        self.dbg.SetAsync(False)

        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// breakpoint_in_main", self.main_source_spec
        )

        frame = thread.GetFrameAtIndex(0)
        options = lldb.SBExpressionOptions()

        # test --language on C++ expression using the SB API's

        # Make sure we can evaluate a C++11 expression.
        val = frame.EvaluateExpression("foo != nullptr")
        self.assertTrue(val.IsValid())
        self.assertSuccess(val.GetError())
        self.DebugSBValue(val)

        # Make sure it still works if language is set to C++11:
        options.SetLanguage(lldb.eLanguageTypeC_plus_plus_11)
        val = frame.EvaluateExpression("foo != nullptr", options)
        self.assertTrue(val.IsValid())
        self.assertSuccess(val.GetError())
        self.DebugSBValue(val)

        # Make sure it fails if language is set to C:
        options.SetLanguage(lldb.eLanguageTypeC)
        val = frame.EvaluateExpression("foo != nullptr", options)
        self.assertTrue(val.IsValid())
        self.assertFalse(val.GetError().Success())

    def test_expr_options_lang(self):
        """These expression language options should work as expected."""
        self.build()

        # Set debugger into synchronous mode
        self.dbg.SetAsync(False)

        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// breakpoint_in_main", self.main_source_spec
        )

        frame = thread.GetFrameAtIndex(0)
        options = lldb.SBExpressionOptions()

        # Make sure we can retrieve `id` variable if language is set to C++11:
        options.SetLanguage(lldb.eLanguageTypeC_plus_plus_11)
        val = frame.EvaluateExpression("id == 0", options)
        self.assertTrue(val.IsValid())
        self.assertSuccess(val.GetError())
        self.DebugSBValue(val)

        # Make sure we can't retrieve `id` variable if language is set to ObjC:
        options.SetLanguage(lldb.eLanguageTypeObjC)
        val = frame.EvaluateExpression("id == 0", options)
        self.assertTrue(val.IsValid())
        self.assertFalse(val.GetError().Success())

    def test_expr_options_language_options(self):
        """Test SetBooleanLanguageOption/GetBooleanLanguageOption SBAPIs"""

        error = lldb.SBError()
        options = lldb.SBExpressionOptions()

        self.assertFalse(options.GetBooleanLanguageOption("foo", error))
        self.assertTrue(error.Fail())
        self.assertFalse(options.GetBooleanLanguageOption("bar", error))
        self.assertTrue(error.Fail())

        self.assertTrue(options.SetBooleanLanguageOption("foo", True).Success())
        self.assertTrue(options.SetBooleanLanguageOption("bar", True).Success())
        self.assertTrue(options.GetBooleanLanguageOption("foo", error))
        self.assertTrue(error.Success())
        self.assertTrue(options.GetBooleanLanguageOption("bar", error))
        self.assertTrue(error.Success())

        self.assertTrue(options.SetBooleanLanguageOption("foo", False).Success())
        self.assertTrue(options.SetBooleanLanguageOption("bar", False).Success())
        self.assertFalse(options.GetBooleanLanguageOption("foo", error))
        self.assertTrue(error.Success())
        self.assertFalse(options.GetBooleanLanguageOption("bar", error))
        self.assertTrue(error.Success())

        self.assertFalse(options.GetBooleanLanguageOption("", error))
        self.assertTrue(error.Fail())
        self.assertTrue(options.SetBooleanLanguageOption("", True).Fail())
        self.assertFalse(options.GetBooleanLanguageOption("", error))
        self.assertTrue(error.Fail())

        self.assertTrue(options.SetBooleanLanguageOption(None, True).Fail())
        self.assertFalse(options.GetBooleanLanguageOption(None, error))
        self.assertTrue(error.Fail())



"""
Test lldb's ability to read and write the LoongArch SIMD registers.
"""

from enum import Enum
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class Mode(Enum):
    LSX = 0
    LASX = 1


class LoongArch64LinuxRegisters(TestBase):
    SHARED_BUILD_TESTCASE = False
    NO_DEBUG_INFO_TESTCASE = True

    def get_build_flags(self, mode):
        cflags = "-march=la464"
        if mode == Mode.LASX:
            cflags += " -DLASX"

        return {"CFLAGS_EXTRAS": cflags}

    def make_simd_value(self, n, mode):
        count = 32 if mode == Mode.LASX else 16
        return "{" + " ".join(["0x{:02x}".format(n)] * count) + "}"

    def check_simd_values(self, value_offset, mode):
        reg_prefix = "xr" if mode == Mode.LASX else "vr"
        for i in range(32):
            self.expect(
                "register read {}{}".format(reg_prefix, i),
                substrs=[self.make_simd_value(i + value_offset, mode)],
            )

    def simd_registers_impl(self, mode):
        self.build(dictionary=self.get_build_flags(mode))
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.c",
            line_number("main.c", "// Set break point at this line."),
            num_expected_locations=1,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.check_simd_values(0, mode)
        self.runCmd("expression write_simd_regs(1)")
        self.check_simd_values(0, mode)

        reg_prefix = "xr" if mode == Mode.LASX else "vr"
        for i in range(32):
            self.runCmd(
                'register write {}{} "{}"'.format(
                    reg_prefix, i, self.make_simd_value(i + 1, mode)
                )
            )

        # Should be visible within lldb.
        self.check_simd_values(1, mode)

        # The program should agree with lldb.
        self.expect("continue", substrs=["exited with status = 0"])

    @skipUnlessArch("loongarch64")
    @skipUnlessPlatform(["linux"])
    def test_lsx(self):
        """Test read/write of LSX registers."""
        if not self.isLoongArchLSX():
            self.skipTest("LSX must be present.")
        self.simd_registers_impl(Mode.LSX)

    @skipUnlessArch("loongarch64")
    @skipUnlessPlatform(["linux"])
    def test_lasx(self):
        """Test read/write of LASX registers."""
        if not self.isLoongArchLASX():
            self.skipTest("LASX must be present.")
        self.simd_registers_impl(Mode.LASX)



"""
Tests forward declarations coming from the `std` module.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import os


class TestCase(TestBase):
    # We only emulate a fake libc++ in this test and don't use the real libc++,
    # but we still add the libc++ category so that this test is only run in
    # test configurations where libc++ is actually supposed to be tested.
    @add_test_categories(["libc++"])
    @skipIfRemote
    @skipIf(compiler=no_match("clang"))
    def test(self):
        self.build()

        sysroot = os.path.join(os.getcwd(), "root")

        # Set the sysroot where our dummy libc++ exists.
        self.runCmd(
            "platform select --sysroot '" + sysroot + "' host", CURRENT_EXECUTABLE_SET
        )

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.import-std-module true")

        # Print the dummy `std::vector`. It only has the dummy member in it
        # so the standard `std::vector` formatter can't format it. Instead use
        # the raw output so LLDB has to show the member variable.
        # Both `std::vector` and the type of the member have forward
        # declarations before their definitions.
        self.expect(
            "expr --raw -- v",
            patterns=[r"\(std::__[^:]*::vector<int>\) \$0 = {", "f = nullptr", "}"],
        )



"""
Test lldb-dap stackTrace request with an extended backtrace thread.
"""

import os

import lldbdap_testcase
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbplatformutil import *


class TestDAP_extendedStackTrace(lldbdap_testcase.DAPTestCaseBase):
    def build_and_run(self, displayExtendedBacktrace=True):
        backtrace_recording_lib = findBacktraceRecordingDylib()
        if not backtrace_recording_lib:
            self.skipTest(
                "Skipped because libBacktraceRecording.dylib was present on the system."
            )

        if not os.path.isfile("/usr/lib/system/introspection/libdispatch.dylib"):
            self.skipTest(
                "Skipped because introspection libdispatch dylib is not present."
            )

        program = self.getBuildArtifact("a.out")

        self.build_and_launch(
            program,
            env=[
                "DYLD_LIBRARY_PATH=/usr/lib/system/introspection",
                "DYLD_INSERT_LIBRARIES=" + backtrace_recording_lib,
            ],
            displayExtendedBacktrace=displayExtendedBacktrace,
        )
        source = "main.m"
        breakpoint = line_number(source, "breakpoint 1")
        lines = [breakpoint]

        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(
            len(breakpoint_ids), len(lines), "expect correct number of breakpoints"
        )

    @skipUnlessDarwin
    def test_stackTrace(self):
        """
        Tests the 'stackTrace' packet on a thread with an extended backtrace.
        """
        self.build_and_run()
        events = self.continue_to_next_stop()

        stackFrames, totalFrames = self.get_stackFrames_and_totalFramesCount(
            threadId=events[0]["body"]["threadId"]
        )
        self.assertGreaterEqual(len(stackFrames), 3, "expect >= 3 frames")
        self.assertEqual(len(stackFrames), totalFrames)
        self.assertEqual(stackFrames[0]["name"], "one")
        self.assertEqual(stackFrames[1]["name"], "two")
        self.assertEqual(stackFrames[2]["name"], "three")

        stackLabels = [
            (i, frame)
            for i, frame in enumerate(stackFrames)
            if frame.get("presentationHint", "") == "label"
        ]
        self.assertEqual(len(stackLabels), 2, "expected two label stack frames")
        self.assertRegex(
            stackLabels[0][1]["name"],
            r"Enqueued from com.apple.root.default-qos \(Thread \d\)",
        )
        self.assertRegex(
            stackLabels[1][1]["name"],
            r"Enqueued from com.apple.main-thread \(Thread \d\)",
        )

        for i, frame in stackLabels:
            # Ensure requesting startFrame+levels across thread backtraces works as expected.
            stackFrames, totalFrames = self.get_stackFrames_and_totalFramesCount(
                threadId=events[0]["body"]["threadId"], startFrame=i - 1, levels=3
            )
            self.assertEqual(len(stackFrames), 3, "expected 3 frames with levels=3")
            self.assertGreaterEqual(
                totalFrames, i + 3, "total frames should include a pagination offset"
            )
            self.assertEqual(stackFrames[1], frame)

            # Ensure requesting startFrame+levels at the beginning of a thread backtraces works as expected.
            stackFrames, totalFrames = self.get_stackFrames_and_totalFramesCount(
                threadId=events[0]["body"]["threadId"], startFrame=i, levels=3
            )
            self.assertEqual(len(stackFrames), 3, "expected 3 frames with levels=3")
            self.assertGreaterEqual(
                totalFrames, i + 3, "total frames should include a pagination offset"
            )
            self.assertEqual(stackFrames[0], frame)

            # Ensure requests with startFrame+levels that end precisely on the last frame includes the totalFrames pagination offset.
            stackFrames, totalFrames = self.get_stackFrames_and_totalFramesCount(
                threadId=events[0]["body"]["threadId"], startFrame=i - 1, levels=1
            )
            self.assertEqual(len(stackFrames), 1, "expected 1 frames with levels=1")
            self.assertGreaterEqual(
                totalFrames, i, "total frames should include a pagination offset"
            )

    @skipUnlessDarwin
    def test_stackTraceWithFormat(self):
        """
        Tests the 'stackTrace' packet on a thread with an extended backtrace using stack trace formats.
        """
        self.build_and_run(displayExtendedBacktrace=False)
        events = self.continue_to_next_stop()

        stackFrames, _ = self.get_stackFrames_and_totalFramesCount(
            threadId=events[0]["body"]["threadId"], format={"includeAll": True}
        )

        stackLabels = [
            (i, frame)
            for i, frame in enumerate(stackFrames)
            if frame.get("presentationHint", "") == "label"
        ]

        self.assertEqual(len(stackLabels), 2, "expected two label stack frames")



import os
from time import sleep

import gdbremote_testcase
import lldbgdbserverutils
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


@skipIfMTE  # MTE security transition shims restrict socket operations.
class TestGdbRemoteAttachWait(gdbremote_testcase.GdbRemoteTestCaseBase):
    SHARED_BUILD_TESTCASE = False

    def _set_up_inferior(self):
        self._exe_to_attach = "%s_%d" % (self.testMethodName, os.getpid())
        self.build(dictionary={"EXE": self._exe_to_attach, "CXX_SOURCES": "main.cpp"})

        if self.getPlatform() != "windows":
            # Use a shim to ensure that the process is ready to be attached from
            # the get-go.
            self._exe_to_run = "shim"
            self._exe_to_attach = lldbutil.install_to_target(
                self, self.getBuildArtifact(self._exe_to_attach)
            )
            self._run_args = [self._exe_to_attach]
            self.build(dictionary={"EXE": self._exe_to_run, "CXX_SOURCES": "shim.cpp"})
        else:
            self._exe_to_run = self._exe_to_attach
            self._run_args = []

    def _launch_inferior(self, args):
        inferior = self.spawnSubprocess(self.getBuildArtifact(self._exe_to_run), args)
        self.assertIsNotNone(inferior)
        self.assertGreater(inferior.pid, 0)
        self.assertTrue(lldbgdbserverutils.process_is_running(inferior.pid, True))
        return inferior

    def _launch_and_wait_for_init(self):
        sync_file_path = lldbutil.append_to_process_working_directory(
            self, "process_ready"
        )
        inferior = self._launch_inferior(self._run_args + [sync_file_path])
        lldbutil.wait_for_file_on_target(self, sync_file_path)
        return inferior

    def _attach_packet(self, packet_type):
        return "read packet: ${};{}#00".format(
            packet_type,
            lldbgdbserverutils.gdbremote_hex_encode_string(self._exe_to_attach),
        )

    @skipIfWindows  # This test is flaky on Windows
    def test_attach_with_vAttachWait(self):
        self._set_up_inferior()

        self.set_inferior_startup_attach_manually()
        server = self.connect_to_debug_monitor()
        self.do_handshake()

        # Launch the first inferior (we shouldn't attach to this one).
        self._launch_and_wait_for_init()

        self.test_sequence.add_log_lines([self._attach_packet("vAttachWait")], True)
        # Run the stream until attachWait.
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Sleep so we're sure that the inferior is launched after we ask for the attach.
        sleep(1)

        # Launch the second inferior (we SHOULD attach to this one).
        inferior_to_attach = self._launch_inferior(self._run_args)

        # Make sure the attach succeeded.
        self.test_sequence.add_log_lines(
            [
                {
                    "direction": "send",
                    "regex": r"^\$T([0-9a-fA-F]{2})[^#]*#[0-9a-fA-F]{2}$",
                    "capture": {1: "stop_signal_hex"},
                },
            ],
            True,
        )
        self.add_process_info_collection_packets()

        # Run the stream sending the response..
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Gather process info response.
        process_info = self.parse_process_info_response(context)
        self.assertIsNotNone(process_info)

        # Ensure the process id matches what we expected.
        pid_text = process_info.get("pid", None)
        self.assertIsNotNone(pid_text)
        reported_pid = int(pid_text, base=16)
        self.assertEqual(reported_pid, inferior_to_attach.pid)

    @skipIfWindows  # This test is flaky on Windows
    def test_launch_before_attach_with_vAttachOrWait(self):
        self._set_up_inferior()

        self.set_inferior_startup_attach_manually()
        server = self.connect_to_debug_monitor()
        self.do_handshake()

        inferior = self._launch_and_wait_for_init()

        # Add attach packets.
        self.test_sequence.add_log_lines(
            [
                # Do the attach.
                self._attach_packet("vAttachOrWait"),
                # Expect a stop notification from the attach.
                {
                    "direction": "send",
                    "regex": r"^\$T([0-9a-fA-F]{2})[^#]*#[0-9a-fA-F]{2}$",
                    "capture": {1: "stop_signal_hex"},
                },
            ],
            True,
        )
        self.add_process_info_collection_packets()

        # Run the stream
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Gather process info response
        process_info = self.parse_process_info_response(context)
        self.assertIsNotNone(process_info)

        # Ensure the process id matches what we expected.
        pid_text = process_info.get("pid", None)
        self.assertIsNotNone(pid_text)
        reported_pid = int(pid_text, base=16)
        self.assertEqual(reported_pid, inferior.pid)

    @skipIfWindows  # This test is flaky on Windows
    def test_launch_after_attach_with_vAttachOrWait(self):
        self._set_up_inferior()

        self.set_inferior_startup_attach_manually()
        server = self.connect_to_debug_monitor()
        self.do_handshake()

        self.test_sequence.add_log_lines([self._attach_packet("vAttachOrWait")], True)
        # Run the stream until attachWait.
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Sleep so we're sure that the inferior is launched after we ask for the attach.
        sleep(1)

        # Launch the inferior.
        inferior = self._launch_inferior(self._run_args)

        # Make sure the attach succeeded.
        self.test_sequence.add_log_lines(
            [
                {
                    "direction": "send",
                    "regex": r"^\$T([0-9a-fA-F]{2})[^#]*#[0-9a-fA-F]{2}$",
                    "capture": {1: "stop_signal_hex"},
                },
            ],
            True,
        )
        self.add_process_info_collection_packets()

        # Run the stream sending the response..
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Gather process info response.
        process_info = self.parse_process_info_response(context)
        self.assertIsNotNone(process_info)

        # Ensure the process id matches what we expected.
        pid_text = process_info.get("pid", None)
        self.assertIsNotNone(pid_text)
        reported_pid = int(pid_text, base=16)
        self.assertEqual(reported_pid, inferior.pid)



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class GenericSetDataFormatterTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False
    TEST_WITH_PDB_DEBUG_INFO = True

    def setUp(self):
        TestBase.setUp(self)
        self.namespace = "std"

    def findVariable(self, name):
        var = self.frame().FindVariable(name)
        self.assertTrue(var.IsValid())
        return var

    def getVariableType(self, name):
        var = self.findVariable(name)
        return var.GetType().GetDisplayTypeName()

    def check(self, var_name, size):
        var = self.findVariable(var_name)
        self.assertEqual(var.GetNumChildren(), size)
        children = []
        for i in range(size):
            child = var.GetChildAtIndex(i)
            children.append(ValueCheck(value=child.GetValue()))
        self.expect_var_path(
            var_name, type=self.getVariableType(var_name), children=children
        )

    def do_test_with_run_command(self):
        """Test that that file and class static variables display correctly."""
        (self.target, process, _, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set break point at this line.", lldb.SBFileSpec("main.cpp", False)
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type filter clear", check=False)
            self.runCmd("type synth clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        ii_type = self.getVariableType("ii")
        self.assertTrue(
            ii_type.startswith(self.namespace + "::set"), "Type: " + ii_type
        )

        self.expect("frame variable ii", substrs=["size=0", "{}"])
        lldbutil.continue_to_breakpoint(process, bkpt)
        self.expect(
            "frame variable ii",
            substrs=[
                "size=6",
                "[0] = 0",
                "[1] = 1",
                "[2] = 2",
                "[3] = 3",
                "[4] = 4",
                "[5] = 5",
            ],
        )

        lldbutil.continue_to_breakpoint(process, bkpt)
        self.check("ii", 7)

        lldbutil.continue_to_breakpoint(process, bkpt)
        self.expect("frame variable ii", substrs=["size=0", "{}"])
        lldbutil.continue_to_breakpoint(process, bkpt)
        self.expect("frame variable ii", substrs=["size=0", "{}"])

        ss_type = self.getVariableType("ss")
        self.assertTrue(
            ii_type.startswith(self.namespace + "::set"), "Type: " + ss_type
        )

        self.expect("frame variable ss", substrs=["size=0", "{}"])
        lldbutil.continue_to_breakpoint(process, bkpt)
        self.expect(
            "frame variable ss",
            substrs=["size=2", '[0] = "a"', '[1] = "a very long string is right here"'],
        )
        lldbutil.continue_to_breakpoint(process, bkpt)
        self.expect(
            "frame variable ss",
            substrs=[
                "size=4",
                '[0] = "a"',
                '[1] = "a very long string is right here"',
                '[2] = "b"',
                '[3] = "c"',
            ],
        )
        self.expect(
            "expression ss",
            substrs=[
                "size=4",
                '[0] = "a"',
                '[1] = "a very long string is right here"',
                '[2] = "b"',
                '[3] = "c"',
            ],
        )
        self.expect("frame variable ss[2]", substrs=[' = "b"'])
        lldbutil.continue_to_breakpoint(process, bkpt)
        self.expect(
            "frame variable ss",
            substrs=[
                "size=3",
                '[0] = "a"',
                '[1] = "a very long string is right here"',
                '[2] = "c"',
            ],
        )
        self.check("ss", 3)

    @add_test_categories(["libstdcxx"])
    def test_with_run_command_libstdcpp(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test_with_run_command()

    @add_test_categories(["libc++"])
    def test_with_run_command_libcpp(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test_with_run_command()

    @add_test_categories(["msvcstl"])
    def test_with_run_command_msvcstl(self):
        # No flags, because the "msvcstl" category checks that the MSVC STL is used by default.
        self.build()
        self.do_test_with_run_command()

    def do_test_ref_and_ptr(self):
        """Test that the data formatters work on ref and ptr."""
        (self.target, process, _, bkpt) = lldbutil.run_to_source_breakpoint(
            self,
            "Stop here to check by ref and ptr.",
            lldb.SBFileSpec("main.cpp", False),
        )
        # The reference should print just like the value:
        self.check("ref", 7)
        self.check("ptr", 7)

        self.expect("frame variable ptr", substrs=["ptr =", "size=7"])
        self.expect("expr ptr", substrs=["size=7"])

    @add_test_categories(["libstdcxx"])
    def test_ref_and_ptr_libstdcpp(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test_ref_and_ptr()

    @add_test_categories(["libc++"])
    def test_ref_and_ptr_libcpp(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test_ref_and_ptr()

    @add_test_categories(["msvcstl"])
    def test_ref_and_ptr_msvcstl(self):
        self.build()
        self.do_test_ref_and_ptr()



"""
Tests that we correctly track AST layout info
(specifically alignment) when moving AST nodes
between several ClangASTImporter instances
(in this case, from a pch chain to executable
to expression AST).
"""

import lldb
import os
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestPchChain(TestBase):
    @add_test_categories(["gmodules"])
    @expectedFailureAll("Chained pch debugging currently not fully supported")
    def test_expr(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.target = self.dbg.CreateTarget(exe)
        self.assertTrue(self.target, VALID_TARGET)
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", 9, num_expected_locations=1
        )

        self.runCmd("run", RUN_SUCCEEDED)

        self.expect(
            "frame variable data",
            substrs=["row = 1", "col = 2", "row = 3", "col = 4", "stride = 5"],
        )

    @add_test_categories(["gmodules"])
    @expectedFailureAll("Chained pch debugging currently not fully supported")
    def test_frame_var(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.target = self.dbg.CreateTarget(exe)
        self.assertTrue(self.target, VALID_TARGET)
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", 9, num_expected_locations=1
        )

        self.runCmd("run", RUN_SUCCEEDED)

        self.expect_expr(
            "data",
            result_type="MatrixData",
            result_children=[
                ValueCheck(
                    name="section",
                    children=[
                        ValueCheck(
                            name="origin",
                            children=[
                                ValueCheck(name="row", value="1"),
                                ValueCheck(name="col", value="2"),
                            ],
                        ),
                        ValueCheck(
                            name="size",
                            children=[
                                ValueCheck(name="row", value="3"),
                                ValueCheck(name="col", value="4"),
                            ],
                        ),
                    ],
                ),
                ValueCheck(name="stride", value="5"),
            ],
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest


class TestIOHandlerProcessSTDIO(PExpectTest):
    NO_DEBUG_INFO_TESTCASE = True

    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIf(oslist=["linux"], archs=["arm$", "aarch64"])
    def test(self):
        self.build()
        self.launch(executable=self.getBuildArtifact("a.out"))
        self.child.sendline("run")

        self.child.send("foo\n")
        self.child.expect_exact("stdout: foo")

        self.child.send("bar\n")
        self.child.expect_exact("stdout: bar")

        self.child.send("baz\n")
        self.child.expect_exact("stdout: baz")

        self.child.sendcontrol("d")
        self.quit()



"""
A test for the curiously recurring template pattern (or CRTP).

Note that the derived class is referenced directly from the parent class in the
test. If this fails then there is a good chance that LLDB tried to eagerly
resolve the definition of the derived class while constructing the base class.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test(self):
        self.build()
        self.createTestTarget()

        # Try using the class in the expression evaluator.
        self.expect_expr(
            "derived",
            result_type="Derived",
            result_children=[
                ValueCheck(name="Base<Derived>"),
                ValueCheck(name="member", value="0"),
            ],
        )

        # Try accessing the members of the class and base class.
        self.expect_expr("derived.pointer", result_type="Derived *")
        self.expect_expr("derived.member", result_type="int", result_value="0")



import lldb


def summary(valobj, dict):
    return f"[{valobj.GetChildAtIndex(0).GetValue()}]"


def __lldb_init_module(debugger, dict):
    typeName = "Box<.*$"
    debugger.HandleCommand(
        'type summary add -x "'
        + typeName
        + '" --python-function '
        + f"{__name__}.summary"
    )



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import transform
from mlir.dialects.transform import gpu


def run(f):
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            sequence = transform.SequenceOp(
                transform.FailurePropagationMode.Propagate,
                [],
                transform.AnyOpType.get(),
            )
            with InsertionPoint(sequence.body):
                f(sequence.bodyTarget)
                transform.YieldOp()
        print("\nTEST:", f.__name__)
        print(module)
    return f


@run
def testMapForallToBlocksCompact(target):
    gpu.MapForallToBlocks(target)
    # CHECK-LABEL: TEST: testMapForallToBlocksCompact
    # CHECK: = transform.gpu.map_forall_to_blocks
    # CHECK-NOT: grid_dims
    # CHECK-SAME: (!transform.any_op) -> !transform.any_op
    # CHECK-NOT: grid_dims


@run
def testMapForallToBlocksTyped(target):
    gpu.MapForallToBlocks(transform.OperationType.get("test.dummy"), target)
    # CHECK-LABEL: TEST: testMapForallToBlocksTyped
    # CHECK: = transform.gpu.map_forall_to_blocks
    # CHECK-SAME: (!transform.any_op) -> !transform.op<"test.dummy">


@run
def testMapForallToBlocksGridDims(target):
    gpu.MapForallToBlocks(target, grid_dims=[4, 2, 1])
    # CHECK-LABEL: TEST: testMapForallToBlocksGridDims
    # CHECK: = transform.gpu.map_forall_to_blocks
    # CHECK-SAME: grid_dims = [4, 2, 1]
    # CHECK-SAME: (!transform.any_op) -> !transform.any_op


@run
def testMapNestedForallToThreadsCompact(target):
    gpu.MapNestedForallToThreads(target)
    # CHECK-LABEL: TEST: testMapNestedForallToThreadsCompact
    # CHECK: transform.gpu.map_nested_forall_to_threads
    # CHECK-SAME: block_dims = []
    # CHECK-SAME: (!transform.any_op) -> !transform.any_op


@run
def testMapNestedForallToThreadsTyped(target):
    gpu.MapNestedForallToThreads(transform.OperationType.get("test.dummy"), target)
    # CHECK-LABEL: TEST: testMapNestedForallToThreadsTyped
    # CHECK: transform.gpu.map_nested_forall_to_threads
    # CHECK-SAME: block_dims = []
    # CHECK-SAME: (!transform.any_op) -> !transform.op<"test.dummy">


@run
def testMapNestedForallToThreadsAttributes(target):
    gpu.MapNestedForallToThreads(
        target, block_dims=[4, 2], warp_size=64, sync_after_distribute=False
    )
    # CHECK-LABEL: TEST: testMapNestedForallToThreadsAttributes
    # CHECK: transform.gpu.map_nested_forall_to_threads
    # CHECK-SAME: block_dims = [4, 2]
    # CHECK-SAME: sync_after_distribute = false
    # CHECK-SAME: warp_size = 64



# RUN: %PYTHON %s | FileCheck %s

import gc
import io
from tempfile import NamedTemporaryFile
from mlir.ir import *
from mlir.dialects.builtin import ModuleOp
from mlir.dialects import arith, func, scf, shape
from mlir.dialects._ods_common import _cext
from mlir.extras import types as T


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0
    return f


def expect_index_error(callback):
    try:
        _ = callback()
        raise RuntimeError("Expected IndexError")
    except IndexError:
        pass


# Verify iterator based traversal of the op/region/block hierarchy.
# CHECK-LABEL: TEST: testTraverseOpRegionBlockIterators
@run
def testTraverseOpRegionBlockIterators():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    module = Module.parse(
        r"""
    func.func @f1(%arg0: i32) -> i32 {
      %1 = "custom.addi"(%arg0, %arg0) : (i32, i32) -> i32
      return %1 : i32
    }
  """,
        ctx,
    )
    op = module.operation
    assert op.context is ctx
    # Note, __nb_signature__ stores the fully-qualified signature - the actual type stub emitted is
    # class RegionSequence(Sequence[Region])
    # CHECK: class RegionSequence(collections.abc.Sequence[mlir._mlir_libs._mlir.ir.Region])
    print(RegionSequence.__nb_signature__)
    # Get the block using iterators off of the named collections.
    regions = list(op.regions[:])
    blocks = list(regions[0].blocks)
    # CHECK: MODULE REGIONS=1 BLOCKS=1
    print(f"MODULE REGIONS={len(regions)} BLOCKS={len(blocks)}")

    # Should verify.
    # CHECK: .verify = True
    print(f".verify = {module.operation.verify()}")

    # Get the blocks from the default collection.
    default_blocks = list(regions[0])
    # They should compare equal regardless of how obtained.
    assert default_blocks == blocks

    # Should be able to get the operations from either the named collection
    # or the block.
    operations = list(blocks[0].operations)
    default_operations = list(blocks[0])
    assert default_operations == operations

    def walk_operations(indent, op):
        for i, region in enumerate(op.regions):
            print(f"{indent}REGION {i}:")
            for j, block in enumerate(region):
                print(f"{indent}  BLOCK {j}:")
                for k, child_op in enumerate(block):
                    print(f"{indent}    OP {k}: {child_op}")
                    walk_operations(indent + "      ", child_op)

    # CHECK: REGION 0:
    # CHECK:   BLOCK 0:
    # CHECK:     OP 0: func
    # CHECK:       REGION 0:
    # CHECK:         BLOCK 0:
    # CHECK:           OP 0: %0 = "custom.addi"
    # CHECK:           OP 1: func.return
    walk_operations("", op)

    # CHECK:    Region iter: <iterator
    # CHECK:     Block iter: <mlir.{{.+}}.BlockIterator
    # CHECK: Operation iter: <mlir.{{.+}}.OperationIterator
    print("   Region iter:", iter(op.regions))
    print("    Block iter:", iter(op.regions[-1]))
    print("Operation iter:", iter(op.regions[-1].blocks[-1]))

    try:
        op.regions[-42]
    except IndexError as e:
        # CHECK: Region OOB: index out of range
        print("Region OOB:", e)
    try:
        op.regions[0].blocks[-42]
    except IndexError as e:
        # CHECK: attempt to access out of bounds block
        print(e)
    try:
        op.regions[0].blocks[0].operations[-42]
    except IndexError as e:
        # CHECK: attempt to access out of bounds operation
        print(e)

    # Verify that iterating a sliced region list yields the correct
    # number of elements (i.e. respects length and step).
    with Location.unknown(ctx):
        op4 = Operation.create("custom.op", regions=4)
        r = op4.regions
        assert len(list(r[:])) == 4
        assert len(list(r[1:])) == 3
        assert len(list(r[::2])) == 2
        assert len(list(r[1:3])) == 2
        assert len(list(r[::-1])) == 4


# Verify index based traversal of the op/region/block hierarchy.
# CHECK-LABEL: TEST: testTraverseOpRegionBlockIndices
@run
def testTraverseOpRegionBlockIndices():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    module = Module.parse(
        r"""
    func.func @f1(%arg0: i32) -> i32 {
      %1 = "custom.addi"(%arg0, %arg0) : (i32, i32) -> i32
      return %1 : i32
    }
  """,
        ctx,
    )

    def walk_operations(indent, op):
        for i in range(len(op.regions)):
            region = op.regions[i]
            print(f"{indent}REGION {i}:")
            for j in range(len(region.blocks)):
                block = region.blocks[j]
                print(f"{indent}  BLOCK {j}:")
                for k in range(len(block.operations)):
                    child_op = block.operations[k]
                    print(f"{indent}    OP {k}: {child_op}")
                    print(
                        f"{indent}    OP {k}: parent {child_op.operation.parent.name}"
                    )
                    walk_operations(indent + "      ", child_op)

    # CHECK: REGION 0:
    # CHECK:   BLOCK 0:
    # CHECK:     OP 0: func
    # CHECK:     OP 0: parent builtin.module
    # CHECK:       REGION 0:
    # CHECK:         BLOCK 0:
    # CHECK:           OP 0: %0 = "custom.addi"
    # CHECK:           OP 0: parent func.func
    # CHECK:           OP 1: func.return
    # CHECK:           OP 1: parent func.func
    walk_operations("", module.operation)


# CHECK-LABEL: TEST: testBlockAndRegionOwners
@run
def testBlockAndRegionOwners():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    module = Module.parse(
        r"""
    builtin.module {
      func.func @f() {
        func.return
      }
    }
  """,
        ctx,
    )

    assert module.operation.regions[0].owner == module.operation
    assert module.operation.regions[0].blocks[0].owner == module.operation

    func = module.body.operations[0]
    assert func.operation.regions[0].owner == func
    assert func.operation.regions[0].blocks[0].owner == func


# CHECK-LABEL: TEST: testBlockArgumentList
@run
def testBlockArgumentList():
    with Context() as ctx:
        module = Module.parse(
            r"""
      func.func @f1(%arg0: i32, %arg1: f64, %arg2: index) {
        return
      }
    """,
            ctx,
        )
        func_op = module.body.operations[0]
        entry_block = func_op.regions[0].blocks[0]
        assert len(entry_block.arguments) == 3
        # CHECK: Argument 0, type i32
        # CHECK: Argument 1, type f64
        # CHECK: Argument 2, type index
        for arg in entry_block.arguments:
            print(f"Argument {arg.arg_number}, type {arg.type}")
            new_type = IntegerType.get_signless(8 * (arg.arg_number + 1))
            arg.set_type(new_type)

        # CHECK: Argument 0, type i8
        # CHECK: Argument 1, type i16
        # CHECK: Argument 2, type i24
        for arg in entry_block.arguments:
            print(f"Argument {arg.arg_number}, type {arg.type}")

        # CHECK: Matched Arg 0, type i8
        # CHECK: Matched Arg 1, type i16
        # CHECK: Matched Arg 2, type i24
        match func_op:
            case func.FuncOp(body=Region(blocks=[Block(arguments=[a0, a1])])):
                assert False
            case func.FuncOp(body=Region(blocks=[Block(arguments=[a0, a1, a2, a3])])):
                assert False
            case func.FuncOp(body=Region(blocks=[Block(arguments=[a0, a1, a2])])):
                print(f"Matched Arg 0, type {a0.type}")
                print(f"Matched Arg 1, type {a1.type}")
                print(f"Matched Arg 2, type {a2.type}")
            case _:
                assert False

        # Check that slicing works for block argument lists.
        # CHECK: Argument 1, type i16
        # CHECK: Argument 2, type i24
        for arg in entry_block.arguments[1:]:
            print(f"Argument {arg.arg_number}, type {arg.type}")

        # Check that we can concatenate slices of argument lists.
        # CHECK: Length: 4
        print("Length: ", len(entry_block.arguments[:2] + entry_block.arguments[1:]))

        # CHECK: Type: i8
        # CHECK: Type: i16
        # CHECK: Type: i24
        for t in entry_block.arguments.types:
            print("Type: ", t)

        # Check that slicing and type access compose.
        # CHECK: Sliced type: i16
        # CHECK: Sliced type: i24
        for t in entry_block.arguments[1:].types:
            print("Sliced type: ", t)

        # Check that slice addition works as expected.
        # CHECK: Argument 2, type i24
        # CHECK: Argument 0, type i8
        restructured = entry_block.arguments[-1:] + entry_block.arguments[:1]
        for arg in restructured:
            print(f"Argument {arg.arg_number}, type {arg.type}")


# CHECK-LABEL: TEST: testOperationOperands
@run
def testOperationOperands():
    with Context() as ctx:
        ctx.allow_unregistered_dialects = True
        module = Module.parse(
            r"""
      func.func @f1(%arg0: i32) {
        %0 = "test.producer"() : () -> i64
        "test.consumer"(%arg0, %0) : (i32, i64) -> ()
        return
      }"""
        )
        func_op = module.body.operations[0]
        entry_block = func_op.regions[0].blocks[0]
        consumer = entry_block.operations[1]
        assert len(consumer.operands) == 2
        # CHECK: Operand 0, type i32
        # CHECK: Operand 1, type i64
        for i, operand in enumerate(consumer.operands):
            print(f"Operand {i}, type {operand.type}")

        match module.body.operations:
            case [
                func.FuncOp(
                    body=Region(
                        blocks=[
                            Block(
                                operations=[
                                    _,
                                    OpView(operands=[o1, o2]) as matched_consumer,
                                    *_,
                                ],
                            ),
                        ],
                    ),
                ),
            ]:
                print(f"Matched Operand 0, type {o1.type}")
                print(f"Matched Operand 1, type {o2.type}")


# CHECK-LABEL: TEST: testOperationOperandsSlice
@run
def testOperationOperandsSlice():
    with Context() as ctx:
        ctx.allow_unregistered_dialects = True
        module = Module.parse(
            r"""
      func.func @f1() {
        %0 = "test.producer0"() : () -> i64
        %1 = "test.producer1"() : () -> i64
        %2 = "test.producer2"() : () -> i64
        %3 = "test.producer3"() : () -> i64
        %4 = "test.producer4"() : () -> i64
        "test.consumer"(%0, %1, %2, %3, %4) : (i64, i64, i64, i64, i64) -> ()
        return
      }"""
        )
        func = module.body.operations[0]
        entry_block = func.regions[0].blocks[0]
        consumer = entry_block.operations[5]
        assert len(consumer.operands) == 5
        for left, right in zip(consumer.operands, consumer.operands[::-1][::-1]):
            assert left == right

        # CHECK: test.producer0
        # CHECK: test.producer1
        # CHECK: test.producer2
        # CHECK: test.producer3
        # CHECK: test.producer4
        full_slice = consumer.operands[:]
        for operand in full_slice:
            print(operand)

        # CHECK: test.producer0
        # CHECK: test.producer1
        first_two = consumer.operands[0:2]
        for operand in first_two:
            print(operand)

        # CHECK: test.producer3
        # CHECK: test.producer4
        last_two = consumer.operands[3:]
        for operand in last_two:
            print(operand)

        # CHECK: test.producer0
        # CHECK: test.producer2
        # CHECK: test.producer4
        even = consumer.operands[::2]
        for operand in even:
            print(operand)

        # CHECK: test.producer2
        fourth = consumer.operands[::2][1::2]
        for operand in fourth:
            print(operand)


# CHECK-LABEL: TEST: testOperationOperandsSet
@run
def testOperationOperandsSet():
    with Context() as ctx, Location.unknown(ctx):
        ctx.allow_unregistered_dialects = True
        module = Module.parse(
            r"""
      func.func @f1() {
        %0 = "test.producer0"() : () -> i64
        %1 = "test.producer1"() : () -> i64
        %2 = "test.producer2"() : () -> i64
        "test.consumer"(%0) : (i64) -> ()
        return
      }"""
        )
        func = module.body.operations[0]
        entry_block = func.regions[0].blocks[0]
        producer1 = entry_block.operations[1]
        producer2 = entry_block.operations[2]
        consumer = entry_block.operations[3]
        assert len(consumer.operands) == 1
        type = consumer.operands[0].type

        # CHECK: test.producer1
        consumer.operands[0] = producer1.result
        print(consumer.operands[0])

        # CHECK: test.producer2
        consumer.operands[-1] = producer2.result
        print(consumer.operands[0])


# CHECK-LABEL: TEST: testDetachedOperation
@run
def testDetachedOperation():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with Location.unknown(ctx):
        i32 = IntegerType.get_signed(32)
        op1 = Operation.create(
            "custom.op1",
            results=[i32, i32],
            regions=1,
            attributes={
                "foo": StringAttr.get("foo_value"),
                "bar": StringAttr.get("bar_value"),
            },
        )
        # CHECK: %0:2 = "custom.op1"() ({
        # CHECK: }) {bar = "bar_value", foo = "foo_value"} : () -> (si32, si32)
        print(op1)

    # TODO: Check successors once enough infra exists to do it properly.


# CHECK-LABEL: TEST: testOperationInsertionPoint
@run
def testOperationInsertionPoint():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    module = Module.parse(
        r"""
    func.func @f1(%arg0: i32) -> i32 {
      %1 = "custom.addi"(%arg0, %arg0) : (i32, i32) -> i32
      return %1 : i32
    }
  """,
        ctx,
    )

    # Create test op.
    with Location.unknown(ctx):
        op1 = Operation.create("custom.op1")
        op2 = Operation.create("custom.op2")

        func = module.body.operations[0]
        entry_block = func.regions[0].blocks[0]
        ip = InsertionPoint.at_block_begin(entry_block)
        ip.insert(op1)
        ip.insert(op2)
        # CHECK: func @f1
        # CHECK: "custom.op1"()
        # CHECK: "custom.op2"()
        # CHECK: %0 = "custom.addi"
        print(module)

    # Trying to add a previously added op should raise.
    try:
        ip.insert(op1)
    except ValueError:
        pass
    else:
        assert False, "expected insert of attached op to raise"


# CHECK-LABEL: TEST: testOperationWithRegion
@run
def testOperationWithRegion():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with Location.unknown(ctx):
        i32 = IntegerType.get_signed(32)
        op1 = Operation.create("custom.op1", regions=1)
        block = op1.regions[0].blocks.append(i32, i32)
        # CHECK: "custom.op1"() ({
        # CHECK: ^bb0(%arg0: si32, %arg1: si32):
        # CHECK:   "custom.terminator"() : () -> ()
        # CHECK: }) : () -> ()
        terminator = Operation.create("custom.terminator")
        ip = InsertionPoint(block)
        ip.insert(terminator)
        print(op1)

        # Now add the whole operation to another op.
        # TODO: Verify lifetime hazard by nulling out the new owning module and
        # accessing op1.
        # TODO: Also verify accessing the terminator once both parents are nulled
        # out.
        module = Module.parse(
            r"""
      func.func @f1(%arg0: i32) -> i32 {
        %1 = "custom.addi"(%arg0, %arg0) : (i32, i32) -> i32
        return %1 : i32
      }
    """
        )
        func = module.body.operations[0]
        entry_block = func.regions[0].blocks[0]
        ip = InsertionPoint.at_block_begin(entry_block)
        ip.insert(op1)
        # CHECK: func @f1
        # CHECK: "custom.op1"()
        # CHECK:   "custom.terminator"
        # CHECK: %0 = "custom.addi"
        print(module)


# CHECK-LABEL: TEST: testOperationResultList
@run
def testOperationResultList():
    ctx = Context()
    module = Module.parse(
        r"""
    func.func @f1() {
      %0:3 = call @f2() : () -> (i32, f64, index)
      call @f3() : () -> ()
      return
    }
    func.func private @f2() -> (i32, f64, index)
    func.func private @f3() -> ()
  """,
        ctx,
    )
    caller = module.body.operations[0]
    call = caller.regions[0].blocks[0].operations[0]
    assert len(call.results) == 3
    # CHECK: Result 0, type i32
    # CHECK: Result 1, type f64
    # CHECK: Result 2, type index
    for res in call.results:
        print(f"Result {res.result_number}, type {res.type}")

    # CHECK: Matched Result r0, type i32
    # CHECK: Matched Result r1, type f64
    # CHECK: Matched Result r2, type index
    match caller:
        case func.FuncOp(
            body=Region(
                blocks=[
                    Block(
                        operations=[OpView(results=[r0, r1, r2]) as matched_call, *_],
                    ),
                ],
            ),
        ):
            assert matched_call == call
            print(f"Matched Result r0, type {r0.type}")
            print(f"Matched Result r1, type {r1.type}")
            print(f"Matched Result r2, type {r2.type}")
        case _:
            assert False

    # CHECK: Result type i32
    # CHECK: Result type f64
    # CHECK: Result type index
    for t in call.results.types:
        print(f"Result type {t}")

    # Out of range
    expect_index_error(lambda: call.results[3])
    expect_index_error(lambda: call.results[-4])

    no_results_call = caller.regions[0].blocks[0].operations[1]
    assert len(no_results_call.results) == 0
    assert no_results_call.results.owner == no_results_call


# CHECK-LABEL: TEST: testOperationResultListSlice
@run
def testOperationResultListSlice():
    with Context() as ctx:
        ctx.allow_unregistered_dialects = True
        module = Module.parse(
            r"""
      func.func @f1() {
        "some.op"() : () -> (i1, i2, i3, i4, i5)
        return
      }
    """
        )
        func = module.body.operations[0]
        entry_block = func.regions[0].blocks[0]
        producer = entry_block.operations[0]

        assert len(producer.results) == 5
        for left, right in zip(producer.results, producer.results[::-1][::-1]):
            assert left == right
            assert left.result_number == right.result_number

        # CHECK: Result 0, type i1
        # CHECK: Result 1, type i2
        # CHECK: Result 2, type i3
        # CHECK: Result 3, type i4
        # CHECK: Result 4, type i5
        full_slice = producer.results[:]
        for res in full_slice:
            print(f"Result {res.result_number}, type {res.type}")

        # CHECK: Result 1, type i2
        # CHECK: Result 2, type i3
        # CHECK: Result 3, type i4
        middle = producer.results[1:4]
        for res in middle:
            print(f"Result {res.result_number}, type {res.type}")

        # CHECK: Result 1, type i2
        # CHECK: Result 3, type i4
        odd = producer.results[1::2]
        for res in odd:
            print(f"Result {res.result_number}, type {res.type}")

        # CHECK: Result 3, type i4
        # CHECK: Result 1, type i2
        inverted_middle = producer.results[-2:0:-2]
        for res in inverted_middle:
            print(f"Result {res.result_number}, type {res.type}")


# CHECK-LABEL: TEST: testOperationAttributes
@run
def testOperationAttributes():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    module = Module.parse(
        r"""
    "some.op"() { some.attribute = 1 : i8,
                  other.attribute = 3.0,
                  dependent = "text" } : () -> ()
  """,
        ctx,
    )
    op = module.body.operations[0]
    assert len(op.attributes) == 3
    iattr = op.attributes["some.attribute"]
    fattr = op.attributes["other.attribute"]
    sattr = op.attributes["dependent"]
    # CHECK: Attribute type i8, value 1
    print(f"Attribute type {iattr.type}, value {iattr.value}")
    # CHECK: Attribute type f64, value 3.0
    print(f"Attribute type {fattr.type}, value {fattr.value}")
    # CHECK: Attribute value text
    print(f"Attribute value {sattr.value}")
    # CHECK: Attribute value b'text'
    print(f"Attribute value {sattr.value_bytes}")

    # Python dict-style iteration
    # We don't know in which order the attributes are stored.
    # CHECK-DAG: dependent
    # CHECK-DAG: other.attribute
    # CHECK-DAG: some.attribute
    for name in op.attributes:
        print(name)

    # Basic dict-like introspection
    # CHECK: True
    print("some.attribute" in op.attributes)
    # CHECK: False
    print("missing" in op.attributes)
    # CHECK: Keys: ['dependent', 'other.attribute', 'some.attribute']
    print("Keys:", sorted(op.attributes.keys()))
    # CHECK: Values count 3
    print("Values count", len(op.attributes.values()))
    # CHECK: Items count 3
    print("Items count", len(op.attributes.items()))

    # Dict() conversion test
    d = {k: v.value for k, v in dict(op.attributes).items()}
    # CHECK: Dict mapping {'dependent': 'text', 'other.attribute': 3.0, 'some.attribute': 1}
    print("Dict mapping", d)

    # Structural pattern matching test using Mapping

    # CHECK: Matched Mapping Attribute 'some.attribute': 1
    # CHECK: Matched Mapping Attribute 'other.attribute': 3.0
    # CHECK: Matched Mapping Attribute 'dependent': text
    match op:
        case OpView(attributes={"does_not_exist": a0}):
            assert False
        case OpView(
            attributes={
                "some.attribute": IntegerAttr(value=some_attr_val) as some_attr,
                "other.attribute": FloatAttr() as other_attr,
                "dependent": StringAttr() as dep_attr,
                **other_attributes,
            }
        ):
            print(f"Matched Mapping Attribute 'some.attribute': {some_attr_val}")
            print(f"Matched Mapping Attribute 'other.attribute': {other_attr.value}")
            print(f"Matched Mapping Attribute 'dependent': {dep_attr.value}")
            assert type(other_attributes) == dict
            assert len(other_attributes) == 0
            assert some_attr == op.attributes.get("some.attribute")
            assert other_attr == op.attributes.get("other.attribute", None)
            assert dep_attr == op.attributes.get("dependent", "Default value")
        case _:
            print("Did not match!")
            assert False

    # Check that exceptions are raised as expected.
    try:
        op.attributes["does_not_exist"]
    except KeyError:
        pass
    else:
        assert False, "expected KeyError on accessing a non-existent attribute"

    try:
        op.attributes[42]
    except IndexError:
        pass
    else:
        assert False, "expected IndexError on accessing an out-of-bounds attribute"

    # Check that exceptions are raised when `get` is used with non-str arg.
    try:
        op.attributes.get(0)
    except TypeError:
        pass
    else:
        assert False, "expected TypeError using int as key for get()"

    try:
        op.attributes.get(0, None)
    except TypeError:
        pass
    else:
        assert False, "expected TypeError using int as key for get()"

    try:
        op.attributes.get([], None)
    except TypeError:
        pass
    else:
        assert False, "expected TypeError using list as key for get()"

    try:
        match op:
            case OpView(attributes={0: a}):
                assert False
    except TypeError:
        pass
    else:
        assert False, "expected TypeError matching OpAttributeMap with int-key "

    # get() does not throw for non existent attributes.
    assert op.attributes.get("does_not_exist") is None
    assert op.attributes.get("does_not_exist", "default_value") == "default_value"


# CHECK-LABEL: TEST: testOperationPrint
@run
def testOperationPrint():
    ctx = Context()
    module = Module.parse(
        r"""
    func.func @f1(%arg0: i32) -> i32 {
      %0 = arith.constant dense<[1, 2, 3, 4]> : tensor<4xi32> loc("nom")
      %1 = arith.constant dense_resource<resource1> : tensor<3xi64>
      return %arg0 : i32
    }

    {-#
      dialect_resources: {
          builtin: {
            resource1: "0x08000000010000000000000002000000000000000300000000000000"
          }
        }
      #-}
  """,
        ctx,
    )

    # Test print to stdout.
    # CHECK: return %arg0 : i32
    # CHECK: resource1: "0x08
    module.operation.print()

    # Test print to text file.
    f = io.StringIO()
    # CHECK: <class 'str'>
    # CHECK: return %arg0 : i32
    module.operation.print(file=f)
    str_value = f.getvalue()
    print(str_value.__class__)
    print(f.getvalue())

    # Test roundtrip to bytecode.
    bytecode_stream = io.BytesIO()
    module.operation.write_bytecode(bytecode_stream, desired_version=1)
    bytecode = bytecode_stream.getvalue()
    assert bytecode.startswith(b"ML\xefR"), "Expected bytecode to start with MLïR"
    with NamedTemporaryFile() as tmpfile:
        module.operation.write_bytecode(str(tmpfile.name), desired_version=1)
        tmpfile.seek(0)
        assert tmpfile.read().startswith(
            b"ML\xefR"
        ), "Expected bytecode to start with MLïR"
    ctx2 = Context()
    module_roundtrip = Module.parse(bytecode, ctx2)
    f = io.StringIO()
    module_roundtrip.operation.print(file=f)
    roundtrip_value = f.getvalue()
    assert str_value == roundtrip_value, "Mismatch after roundtrip bytecode"

    # Test print to binary file.
    f = io.BytesIO()
    # CHECK: <class 'bytes'>
    # CHECK: return %arg0 : i32
    module.operation.print(file=f, binary=True)
    bytes_value = f.getvalue()
    print(bytes_value.__class__)
    print(bytes_value)

    # Test print local_scope.
    # CHECK: constant dense<[1, 2, 3, 4]> : tensor<4xi32> loc("nom")
    module.operation.print(enable_debug_info=True, use_local_scope=True)
    # CHECK: %nom = arith.constant dense<[1, 2, 3, 4]> : tensor<4xi32>
    module.operation.print(use_name_loc_as_prefix=True, use_local_scope=True)

    # Test printing using state.
    state = AsmState(module.operation)
    # CHECK: constant dense<[1, 2, 3, 4]> : tensor<4xi32>
    module.operation.print(state)

    # Test print with options.
    # CHECK: value = dense_resource<__elided__> : tensor<4xi32>
    # CHECK: "func.return"(%arg0) : (i32) -> () -:5:7
    # CHECK-NOT: resource1: "0x08
    module.operation.print(
        large_elements_limit=2,
        enable_debug_info=True,
        pretty_debug_info=True,
        print_generic_op_form=True,
        use_local_scope=True,
    )

    # Test print with skip_regions option
    # CHECK: func.func @f1(%arg0: i32) -> i32
    # CHECK-NOT: func.return
    module.body.operations[0].print(
        skip_regions=True,
    )

    # Test print with large_resource_limit.
    # CHECK: func.func @f1(%arg0: i32) -> i32
    # CHECK-NOT: resource1: "0x08
    module.operation.print(large_resource_limit=2)

    # Test large_elements_limit has no effect on resource string
    # CHECK: func.func @f1(%arg0: i32) -> i32
    # CHECK: resource1: "0x08
    module.operation.print(large_elements_limit=2)


# CHECK-LABEL: TEST: testKnownOpView
@run
def testKnownOpView():
    with Context(), Location.unknown():
        Context.current.allow_unregistered_dialects = True
        module = Module.parse(
            r"""
      %1 = "custom.f32"() : () -> f32
      %2 = "custom.f32"() : () -> f32
      %3 = arith.addf %1, %2 : f32
      %4 = arith.constant 0 : i32
    """
        )
        print(module)

        # addf should map to a known OpView class in the arithmetic dialect.
        # We know the OpView for it defines an 'lhs' attribute.
        addf = module.body.operations[2]
        # CHECK: <mlir.dialects._arith_ops_gen.AddFOp object
        print(repr(addf))
        # CHECK: "custom.f32"()
        print(addf.lhs)

        # One of the custom ops should resolve to the default OpView.
        custom = module.body.operations[0]
        # CHECK: OpView object
        print(repr(custom))

        # Check again to make sure negative caching works.
        custom = module.body.operations[0]
        # CHECK: OpView object
        print(repr(custom))

        # constant should map to an extension OpView class in the arithmetic dialect.
        constant = module.body.operations[3]
        # CHECK: <mlir.dialects.arith.ConstantOp object
        print(repr(constant))
        # Checks that the arith extension is being registered successfully
        # (literal_value is a property on the extension class but not on the default OpView).
        # CHECK: literal value 0
        print("literal value", constant.literal_value)

        # Checks that "late" registration/replacement (i.e., post all module loading/initialization)
        # is working correctly.
        @_cext.register_operation(arith._Dialect, replace=True)
        class ConstantOp(arith.ConstantOp):
            def __init__(self, result, value, *, loc=None, ip=None):
                if isinstance(value, int):
                    super().__init__(IntegerAttr.get(result, value), loc=loc, ip=ip)
                elif isinstance(value, float):
                    super().__init__(FloatAttr.get(result, value), loc=loc, ip=ip)
                else:
                    super().__init__(value, loc=loc, ip=ip)

        constant = module.body.operations[3]
        # CHECK: <__main__.testKnownOpView.<locals>.ConstantOp object
        print(repr(constant))


# CHECK-LABEL: TEST: testFailedGenericOperationCreationReportsError
@run
def testFailedGenericOperationCreationReportsError():
    with Context(), Location.unknown():
        c0 = shape.const_shape([])
        c1 = shape.const_shape([1, 2, 3])
        try:
            shape.MeetOp.build_generic(operands=[c0, c1])
        except MLIRError as e:
            # CHECK: unequal shape cardinality
            print(e)
        else:
            assert False, "Expected exception"


# CHECK-LABEL: TEST: testSingleResultProperty
@run
def testSingleResultProperty():
    with Context(), Location.unknown():
        Context.current.allow_unregistered_dialects = True
        module = Module.parse(
            r"""
      "custom.no_result"() : () -> ()
      %0:2 = "custom.two_result"() : () -> (f32, f32)
      %1 = "custom.one_result"() : () -> f32
    """
        )
        print(module)

    try:
        module.body.operations[0].result
    except ValueError as e:
        # CHECK: Cannot call .result on operation custom.no_result which has 0 results
        print(e)
    else:
        assert False, "Expected exception"

    try:
        module.body.operations[1].result
    except ValueError as e:
        # CHECK: Cannot call .result on operation custom.two_result which has 2 results
        print(e)
    else:
        assert False, "Expected exception"

    # CHECK: %1 = "custom.one_result"() : () -> f32
    print(module.body.operations[2])


def create_invalid_operation():
    # This module has two region and is invalid verify that we fallback
    # to the generic printer for safety.
    op = Operation.create("builtin.module", regions=2)
    op.regions[0].blocks.append()
    return op


# CHECK-LABEL: TEST: testInvalidOperationStrSoftFails
@run
def testInvalidOperationStrSoftFails():
    ctx = Context()
    with Location.unknown(ctx):
        invalid_op = create_invalid_operation()
        # Verify that we fallback to the generic printer for safety.
        # CHECK: "builtin.module"() ({
        # CHECK: }) : () -> ()
        print(invalid_op)
        try:
            invalid_op.verify()
        except MLIRError as e:
            # CHECK: Exception: <
            # CHECK:   Verification failed:
            # CHECK:   error: unknown: 'builtin.module' op requires one region
            # CHECK:    note: unknown: see current operation:
            # CHECK:     "builtin.module"() ({
            # CHECK:     ^bb0:
            # CHECK:     }, {
            # CHECK:     }) : () -> ()
            # CHECK: >
            print(f"Exception: <{e}>")


# CHECK-LABEL: TEST: testInvalidModuleStrSoftFails
@run
def testInvalidModuleStrSoftFails():
    ctx = Context()
    with Location.unknown(ctx):
        module = Module.create()
        with InsertionPoint(module.body):
            invalid_op = create_invalid_operation()
        # Verify that we fallback to the generic printer for safety.
        # CHECK: "builtin.module"() ({
        # CHECK: }) : () -> ()
        print(module)


# CHECK-LABEL: TEST: testInvalidOperationGetAsmBinarySoftFails
@run
def testInvalidOperationGetAsmBinarySoftFails():
    ctx = Context()
    with Location.unknown(ctx):
        invalid_op = create_invalid_operation()
        # Verify that we fallback to the generic printer for safety.
        # CHECK: b'"builtin.module"() ({\n^bb0:\n}, {\n}) : () -> ()\n'
        print(invalid_op.get_asm(binary=True))


# CHECK-LABEL: TEST: testCreateWithInvalidAttributes
@run
def testCreateWithInvalidAttributes():
    ctx = Context()
    with Location.unknown(ctx):
        try:
            Operation.create(
                "builtin.module", attributes={None: StringAttr.get("name")}
            )
        except Exception as e:
            # CHECK: Invalid attribute key (not a string) when attempting to create the operation "builtin.module"
            print(e)
        try:
            Operation.create("builtin.module", attributes={42: StringAttr.get("name")})
        except Exception as e:
            # CHECK: Invalid attribute key (not a string) when attempting to create the operation "builtin.module"
            print(e)
        try:
            Operation.create("builtin.module", attributes={"some_key": ctx})
        except Exception as e:
            # CHECK: Invalid attribute value for the key "some_key" when attempting to create the operation "builtin.module"
            print(e)
        try:
            Operation.create("builtin.module", attributes={"some_key": None})
        except Exception as e:
            # CHECK: Found an invalid (`None`?) attribute value for the key "some_key" when attempting to create the operation "builtin.module"
            print(e)


# CHECK-LABEL: TEST: testOperationName
@run
def testOperationName():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    module = Module.parse(
        r"""
    %0 = "custom.op1"() : () -> f32
    %1 = "custom.op2"() : () -> i32
    %2 = "custom.op1"() : () -> f32
  """,
        ctx,
    )

    # CHECK: custom.op1
    # CHECK: custom.op2
    # CHECK: custom.op1
    for op in module.body.operations:
        print(op.operation.name)


# CHECK-LABEL: TEST: testCapsuleConversions
@run
def testCapsuleConversions():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with Location.unknown(ctx):
        m = Operation.create("custom.op1").operation
        m_capsule = m._CAPIPtr
        assert '"mlir.ir.Operation._CAPIPtr"' in repr(m_capsule)
        m2 = Operation._CAPICreate(m_capsule)
        assert m2 is not m
        assert m2 == m
        # Gc and verify destructed.
        m = None
        m_capsule = None
        m2 = None
        gc.collect()


# CHECK-LABEL: TEST: testOperationErase
@run
def testOperationErase():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with Location.unknown(ctx):
        m = Module.create()
        with InsertionPoint(m.body):
            op = Operation.create("custom.op1")

            # CHECK: "custom.op1"
            print(m)

            op.operation.erase()

            # CHECK-NOT: "custom.op1"
            print(m)

            # Ensure we can create another operation
            Operation.create("custom.op2")


# CHECK-LABEL: TEST: testOperationClone
@run
def testOperationClone():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with Location.unknown(ctx):
        m = Module.create()
        with InsertionPoint(m.body):
            op = Operation.create("custom.op1")

            # CHECK: "custom.op1"
            print(m)

            clone = op.operation.clone()
            op.operation.erase()

            # CHECK: "custom.op1"
            print(m)


# CHECK-LABEL: TEST: testOperationLoc
@run
def testOperationLoc():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with ctx:
        loc = Location.name("loc")
        op = Operation.create("custom.op", loc=loc)
        assert op.location == loc
        assert op.operation.location == loc

        another_loc = Location.name("another_loc")
        op.location = another_loc
        assert op.location == another_loc
        assert op.operation.location == another_loc
        # CHECK: loc("another_loc")
        print(op.location)


# CHECK-LABEL: TEST: testModuleMerge
@run
def testModuleMerge():
    with Context():
        m1 = Module.parse("func.func private @foo()")
        m2 = Module.parse(
            """
      func.func private @bar()
      func.func private @qux()
    """
        )
        foo = m1.body.operations[0]
        bar = m2.body.operations[0]
        qux = m2.body.operations[1]
        assert bar.is_before_in_block(qux)
        bar.move_before(foo)
        assert bar.is_before_in_block(foo)
        qux.move_after(foo)
        assert bar.is_before_in_block(qux)
        assert foo.is_before_in_block(qux)

        # CHECK: module
        # CHECK: func private @bar
        # CHECK: func private @foo
        # CHECK: func private @qux
        print(m1)

        # CHECK: module {
        # CHECK-NEXT: }
        print(m2)


# CHECK-LABEL: TEST: testAppendMoveFromAnotherBlock
@run
def testAppendMoveFromAnotherBlock():
    with Context():
        m1 = Module.parse("func.func private @foo()")
        m2 = Module.parse("func.func private @bar()")
        func = m1.body.operations[0]
        m2.body.append(func)

        # CHECK: module
        # CHECK: func private @bar
        # CHECK: func private @foo

        print(m2)
        # CHECK: module {
        # CHECK-NEXT: }
        print(m1)


# CHECK-LABEL: TEST: testDetachFromParent
@run
def testDetachFromParent():
    with Context():
        m1 = Module.parse("func.func private @foo()")
        func = m1.body.operations[0].detach_from_parent()
        # CHECK: func.attached=False
        print(f"{func.attached=}")

        try:
            func.detach_from_parent()
        except ValueError as e:
            if "has no parent" not in str(e):
                raise
        else:
            assert False, "expected ValueError when detaching a detached operation"

        print(m1)
        # CHECK-NOT: func private @foo


# CHECK-LABEL: TEST: testOperationHash
@run
def testOperationHash():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with ctx, Location.unknown():
        op = Operation.create("custom.op1")
        assert hash(op) == hash(op.operation)

        module = Module.create()
        with InsertionPoint(module.body):
            op2 = Operation.create("custom.op2")
            custom_op2 = module.body.operations[0]
            assert hash(op2) == hash(custom_op2)


# CHECK-LABEL: TEST: testOperationParse
@run
def testOperationParse():
    with Context() as ctx:
        ctx.allow_unregistered_dialects = True

        # Generic operation parsing.
        m = Operation.parse("module {}")
        o = Operation.parse('"test.foo"() : () -> ()')
        assert isinstance(m, ModuleOp)
        assert type(o) is OpView

        # Parsing specific operation.
        m = ModuleOp.parse("module {}")
        assert isinstance(m, ModuleOp)
        try:
            ModuleOp.parse('"test.foo"() : () -> ()')
        except MLIRError as e:
            # CHECK: error: Expected a 'builtin.module' op, got: 'test.foo'
            print(f"error: {e}")
        else:
            assert False, "expected error"

        o = Operation.parse('"test.foo"() : () -> ()', source_name="my-source-string")
        # CHECK: op_with_source_name: "test.foo"() : () -> () loc("my-source-string":1:1)
        print(
            f"op_with_source_name: {o.get_asm(enable_debug_info=True, use_local_scope=True)}"
        )


# CHECK-LABEL: TEST: testOpWalk
@run
def testOpWalk():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    module = Module.parse(
        r"""
    builtin.module {
      func.func @f() {
        func.return
      }
    }
  """,
        ctx,
    )

    def callback(op):
        print(op.name)
        return WalkResult.ADVANCE

    # Test post-order walk (default).
    # CHECK-NEXT:  Post-order
    # CHECK-NEXT:  func.return
    # CHECK-NEXT:  func.func
    # CHECK-NEXT:  builtin.module
    print("Post-order")
    module.operation.walk(callback)

    # Test pre-order walk.
    # CHECK-NEXT:  Pre-order
    # CHECK-NEXT:  builtin.module
    # CHECK-NEXT:  func.fun
    # CHECK-NEXT:  func.return
    print("Pre-order")
    module.operation.walk(callback, WalkOrder.PRE_ORDER)

    # Test interrput.
    # CHECK-NEXT:  Interrupt post-order
    # CHECK-NEXT:  func.return
    print("Interrupt post-order")

    def callback(op):
        print(op.name)
        return WalkResult.INTERRUPT

    module.operation.walk(callback)

    # Test skip.
    # CHECK-NEXT:  Skip pre-order
    # CHECK-NEXT:  builtin.module
    print("Skip pre-order")

    def callback(op):
        print(op.name)
        return WalkResult.SKIP

    module.operation.walk(callback, WalkOrder.PRE_ORDER)

    # Test exception.
    # CHECK: Exception
    # CHECK-NEXT: func.return
    # CHECK-NEXT: Exception raised
    print("Exception")

    def callback(op):
        print(op.name)
        raise ValueError
        return WalkResult.ADVANCE

    try:
        module.operation.walk(callback)
    except RuntimeError:
        print("Exception raised")

    # Test op_class filter: only visits ops of the requested type.
    module = Module.parse(
        r"""
    module {
      func.func @f() {
        func.return
      }
      func.func @g() {
        func.return
      }
      arith.constant dense<0> : tensor<i32>
    }
  """,
        ctx,
    )

    # CHECK-NEXT: only FuncOp visited: True
    only_funcs = True

    def check_type(op):
        nonlocal only_funcs
        if not isinstance(op.opview, func.FuncOp):
            only_funcs = False
        return WalkResult.ADVANCE

    module.operation.walk(check_type, op_class=func.FuncOp)
    print(f"only FuncOp visited: {only_funcs}")

    # CHECK-NEXT: interrupted after: 1
    seen = []

    def stop_after_first(op):
        seen.append(op.opview)
        return WalkResult.INTERRUPT

    module.operation.walk(stop_after_first, op_class=func.FuncOp)
    print(f"interrupted after: {len(seen)}")

    # CHECK-NEXT: never called: True
    called = False

    def should_not_run(op):
        nonlocal called
        called = True
        return WalkResult.ADVANCE

    module.operation.walk(should_not_run, op_class=scf.ForOp)
    print(f"never called: {not called}")

    # CHECK-NEXT: collected func.FuncOp: ['"f"', '"g"']
    collected = []

    def collect(op):
        collected.append(op.opview)
        return WalkResult.ADVANCE

    module.operation.walk(collect, op_class=func.FuncOp)
    assert all(isinstance(r, func.FuncOp) for r in collected)
    print(f"collected func.FuncOp: {[str(r.name) for r in collected]}")

    # Test op_class with walk_order: pre-order visits FuncOps in source order.
    # CHECK-NEXT: pre-order FuncOp names: ['"f"', '"g"']
    collected.clear()
    module.operation.walk(collect, WalkOrder.PRE_ORDER, op_class=func.FuncOp)
    assert all(isinstance(r, func.FuncOp) for r in collected)
    print(f"pre-order FuncOp names: {[str(r.name) for r in collected]}")


# CHECK-LABEL: TEST: testOpReplaceUsesWith
@run
def testOpReplaceUsesWith():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with Location.unknown(ctx):
        m = Module.create()
        i32 = IntegerType.get_signless(32)
        with InsertionPoint(m.body):
            value = Operation.create("custom.op1", results=[i32]).results[0]
            value2 = Operation.create("custom.op2", results=[i32]).results[0]
            op = Operation.create("custom.op3", operands=[value])
            op2 = Operation.create("custom.op4", operands=[value])
            op.replace_uses_of_with(value, value2)

    assert len(list(value.uses)) == 1

    # CHECK: Use owner: "custom.op4"
    # CHECK: Use operand_number: 0
    for use in value.uses:
        assert use.owner in [op2]
        print(f"Use owner: {use.owner}")
        print(f"Use operand_number: {use.operand_number}")

    assert len(list(value2.uses)) == 1

    # CHECK: Use owner: "custom.op3"
    # CHECK: Use operand_number: 0
    for use in value2.uses:
        assert use.owner in [op]
        print(f"Use owner: {use.owner}")
        print(f"Use operand_number: {use.operand_number}")


# CHECK-LABEL: TEST: testGetOwnerConcreteOpview
@run
def testGetOwnerConcreteOpview():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            a = arith.ConstantOp(value=42, result=IntegerType.get_signless(32))
            r = arith.AddIOp(a, a, overflowFlags=arith.IntegerOverflowFlags.nsw)
            for u in a.result.uses:
                assert isinstance(u.owner, arith.AddIOp)


# CHECK-LABEL: TEST: testIndexSwitch
@run
def testIndexSwitch():
    with Context() as ctx, Location.unknown():
        i32 = T.i32()
        module = Module.create()
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(T.index())
            def index_switch(index):
                c1 = arith.constant(i32, 1)
                switch_op = scf.IndexSwitchOp(results=[i32], arg=index, cases=range(3))

                assert len(switch_op.regions) == 4
                assert len(switch_op.regions[2:]) == 2
                assert len([i for i in switch_op.regions[2:]]) == 2
                assert len(switch_op.caseRegions) == 3
                assert len([i for i in switch_op.caseRegions]) == 3
                assert len(switch_op.caseRegions[1:]) == 2
                assert len([i for i in switch_op.caseRegions[1:]]) == 2


# CHECK-LABEL: TEST: testGetParentOfType
@run
def testGetParentOfType():
    with Context() as ctx, Location.unknown():
        ctx.allow_unregistered_dialects = True
        idx = IndexType.get()
        # Build: func.func -> scf.for -> custom.base_op
        func_op: func.FuncOp = func.FuncOp("test_fn", ([], []))
        with InsertionPoint(func_op.add_entry_block()):
            lower_bound = arith.ConstantOp(idx, 0)
            upper_bound = arith.ConstantOp(idx, 10)
            step = arith.ConstantOp(idx, 1)
            for_op: scf.ForOp = scf.ForOp(lower_bound, upper_bound, step)
            with InsertionPoint(for_op.body):
                base_op: Operation = Operation.create("custom.base_op")
                scf.YieldOp([])
            func.ReturnOp([])

        # CHECK: get_parent_of_type detached->func.func: None
        detached: Operation = Operation.create("custom.detached")
        res = get_parent_of_type(detached, func.FuncOp)
        print(f"get_parent_of_type detached->func.func: {res}")
        assert res is None

        # CHECK: get_parent_of_type base_op->func.func: func.func
        res = get_parent_of_type(base_op, func.FuncOp)
        print(f"get_parent_of_type base_op->func.func: {res.operation.name}")
        assert isinstance(res, func.FuncOp)

        # CHECK: get_parent_of_type func_op->func.func: None
        res = get_parent_of_type(func_op, func.FuncOp)
        print(f"get_parent_of_type func_op->func.func: {res}")
        assert res is None

        # CHECK: get_parent_of_type base_op->scf.if: None
        res = get_parent_of_type(base_op, scf.IfOp)
        print(f"get_parent_of_type base_op->scf.if: {res}")
        assert res is None

        try:
            get_parent_of_type(base_op, int)
            assert False, "expected TypeError"
        except TypeError:
            pass


# CHECK-LABEL: TEST: test_get_ops_of_type
@run
def test_get_ops_of_type():
    with Context(), Location.unknown():
        module = Module.parse(
            r"""
    module {
      func.func @f() {
        func.return
      }
      func.func @g() {
        func.return
      }
    }
  """
        )

        # CHECK: get_ops_of_type func.func count: 2
        results = get_ops_of_type(module, func.FuncOp)
        print(f"get_ops_of_type func.func count: {len(results)}")
        assert len(results) == 2
        assert all(isinstance(r, func.FuncOp) for r in results)

        # CHECK: get_ops_of_type scf.for count: 0
        results = get_ops_of_type(module, scf.ForOp)
        print(f"get_ops_of_type scf.for count: {len(results)}")
        assert len(results) == 0

        # CHECK: get_ops_of_type func_op->func.ReturnOp count: 1
        # Accepts OpView as root.
        func_op = get_ops_of_type(module, func.FuncOp)[0]
        results = get_ops_of_type(func_op, func.ReturnOp)
        print(f"get_ops_of_type func_op->func.ReturnOp count: {len(results)}")
        assert len(results) == 1
        assert isinstance(results[0], func.ReturnOp)

        # CHECK: get_ops_of_type no filter count: 5
        # No op_class collects all ops.
        results = get_ops_of_type(module)
        print(f"get_ops_of_type no filter count: {len(results)}")
        assert len(results) == 5
        assert any(isinstance(r, func.FuncOp) for r in results)
        assert any(isinstance(r, func.ReturnOp) for r in results)



# -*- Python -*-

import json
import os
import platform
import re
import shutil
import site
import subprocess
import sys

import lit.formats
from lit.llvm import llvm_config
from lit.llvm.subst import FindTool
from lit.llvm.subst import ToolSubst

site.addsitedir(os.path.dirname(__file__))
from helper import toolchain

# name: The name of this test suite.
config.name = "lldb-shell"

# testFormat: The test format to use to interpret tests.
# We prefer the lit internal shell which provides a better user experience on
# failures and is faster unless the user explicitly disables it with
# LIT_USE_INTERNAL_SHELL=0 env var.
use_lit_shell = True
lit_shell_env = os.environ.get("LIT_USE_INTERNAL_SHELL")
if lit_shell_env:
    use_lit_shell = lit.util.pythonize_bool(lit_shell_env)

config.test_format = toolchain.ShTestLldb(not use_lit_shell)

# suffixes: A list of file extensions to treat as test files. This is overriden
# by individual lit.local.cfg files in the test subdirectories.
config.suffixes = [".test", ".cpp", ".s", ".m", ".ll", ".c"]

# excludes: A list of directories to exclude from the testsuite. The 'Inputs'
# subdirectories contain auxiliary inputs for various tests in their parent
# directories.
config.excludes = ["Inputs", "CMakeLists.txt", "README.txt", "LICENSE.txt"]

# test_source_root: The root path where tests are located.
config.test_source_root = os.path.dirname(__file__)

# test_exec_root: The root path where tests should be run.
config.test_exec_root = os.path.join(config.lldb_obj_root, "test", "Shell")

# Propagate environment vars.
llvm_config.with_system_environment(
    [
        "FREEBSD_LEGACY_PLUGIN",
        "HOME",
        "TEMP",
        "TMP",
        "XDG_CACHE_HOME",
    ]
)

# Enable sanitizer runtime flags.
if config.llvm_use_sanitizer:
    config.environment["ASAN_OPTIONS"] = "detect_stack_use_after_return=1"
    config.environment["TSAN_OPTIONS"] = "halt_on_error=1"
    config.environment["MallocNanoZone"] = "0"

if config.lldb_platform_url and config.cmake_sysroot and config.enable_remote:
    if re.match(r".*-linux.*", config.target_triple):
        config.available_features.add("remote-linux")
else:
    # After this, enable_remote == True iff remote testing is going to be used.
    config.enable_remote = False

llvm_config.use_default_substitutions()
toolchain.use_lldb_substitutions(config)
toolchain.use_support_substitutions(config)

if re.match(r"^arm(hf.*-linux)|(.*-linux-gnuabihf)", config.target_triple):
    config.available_features.add("armhf-linux")

if re.match(r".*-(windows|mingw32)", config.target_triple):
    config.available_features.add("target-windows")

if re.match(r".*-(windows-msvc)$", config.target_triple):
    config.available_features.add("windows-msvc")

if re.match(r".*-(windows-gnu|mingw32)$", config.target_triple):
    config.available_features.add("windows-gnu")


def calculate_arch_features(arch_string):
    # This will add a feature such as x86, arm, mips, etc for each built
    # target
    features = []
    for arch in arch_string.split():
        features.append(arch.lower())
    return features


# Run llvm-config and add automatically add features for whether we have
# assertions enabled, whether we are in debug mode, and what targets we
# are built for.
llvm_config.feature_config(
    [
        ("--assertion-mode", {"ON": "asserts"}),
        ("--build-mode", {"DEBUG": "debug"}),
        ("--targets-built", calculate_arch_features),
    ]
)

# Clean the module caches in the test build directory. This is necessary in an
# incremental build whenever clang changes underneath, so doing it once per
# lit.py invocation is close enough.
for cachedir in [config.clang_module_cache, config.lldb_module_cache]:
    if os.path.isdir(cachedir):
        lit_config.note("Deleting module cache at %s." % cachedir)
        shutil.rmtree(cachedir)

# Set a default per-test timeout of 10 minutes. Setting a timeout per test
# requires that killProcessAndChildren() is supported on the platform and
# lit complains if the value is set but it is not supported.
supported, errormsg = lit_config.maxIndividualTestTimeIsSupported
if supported:
    lit_config.maxIndividualTestTime = 600
else:
    lit_config.warning("Could not set a default per-test timeout. " + errormsg)


# If running tests natively, check for CPU features needed for some tests.

if "native" in config.available_features:
    cpuid_exe = lit.util.which("lit-cpuid", config.lldb_tools_dir)
    if cpuid_exe is None:
        lit_config.warning(
            "lit-cpuid not found, tests requiring CPU extensions will be skipped"
        )
    else:
        out, err, exitcode = lit.util.executeCommand([cpuid_exe])
        if exitcode == 0:
            for x in out.split():
                config.available_features.add("native-cpu-%s" % x)
        else:
            lit_config.warning("lit-cpuid failed: %s" % err)

if config.lldb_enable_python:
    config.available_features.add("python")

if getattr(config, "lldb_enable_mte", False):
    config.available_features.add("lldb-mte")

if config.lldb_enable_lua:
    config.available_features.add("lua")

if config.lldb_enable_lzma:
    config.available_features.add("lzma")

if shutil.which("xz") is not None:
    config.available_features.add("xz")

if config.lldb_system_debugserver:
    config.available_features.add("system-debugserver")

if config.have_lldb_server:
    config.available_features.add("lldb-server")

if config.objc_gnustep_dir:
    config.available_features.add("objc-gnustep")
    if platform.system() == "Windows":
        # objc.dll must be in PATH since Windows has no rpath
        config.environment["PATH"] = os.path.pathsep.join(
            (
                os.path.join(config.objc_gnustep_dir, "lib"),
                config.environment.get("PATH", ""),
            )
        )

if config.have_dia_sdk:
    config.available_features.add("diasdk")

# NetBSD permits setting dbregs either if one is root
# or if user_set_dbregs is enabled
can_set_dbregs = True
if platform.system() == "NetBSD" and os.geteuid() != 0:
    try:
        output = (
            subprocess.check_output(
                ["/sbin/sysctl", "-n", "security.models.extensions.user_set_dbregs"]
            )
            .decode()
            .strip()
        )
        if output != "1":
            can_set_dbregs = False
    except subprocess.CalledProcessError:
        can_set_dbregs = False
if can_set_dbregs:
    config.available_features.add("dbregs-set")

if "LD_PRELOAD" in os.environ:
    config.available_features.add("ld_preload-present")

# Determine if a specific version of Xcode's linker contains a bug. We want to
# skip affected tests if they contain this bug.
if platform.system() == "Darwin":
    try:
        raw_version_details = subprocess.check_output(
            ("xcrun", "ld", "-version_details")
        )
        version_details = json.loads(raw_version_details)
        version = version_details.get("version", "0")
        version_tuple = tuple(int(x) for x in version.split("."))
        if (1000,) <= version_tuple <= (1109,):
            config.available_features.add("ld_new-bug")
    except:
        pass

# Some shell tests dynamically link with python.dll and need to know the
# location of the Python libraries. This ensures that we use the same
# version of Python that was used to build lldb to run our tests.
config.environment["PYTHONHOME"] = config.python_root_dir
config.environment["PATH"] = os.path.pathsep.join(
    (config.python_root_dir, config.environment.get("PATH", ""))
)



"""
Test lldb-dap start-debugging reverse requests.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbdap_testcase


class TestDAP_startDebugging(lldbdap_testcase.DAPTestCaseBase):
    def test_startDebugging(self):
        """
        Tests the "startDebugging" reverse request. It makes sure that the IDE can
        start a child debug session.
        """
        program = self.getBuildArtifact("a.out")
        source = "main.c"
        self.build_and_launch(program)

        breakpoint_line = line_number(source, "// breakpoint")

        self.set_source_breakpoints(source, [breakpoint_line])
        self.continue_to_next_stop()
        self.dap_server.request_evaluate(
            "`lldb-dap start-debugging attach '{\"pid\":321}'", context="repl"
        )

        self.continue_to_exit()

        self.assertEqual(
            len(self.dap_server.reverse_requests),
            1,
            "make sure we got a reverse request",
        )

        request = self.dap_server.reverse_requests[0]
        self.assertEqual(request["arguments"]["configuration"]["pid"], 321)
        self.assertEqual(request["arguments"]["request"], "attach")

    def test_startDebugging_debugger_reuse(self):
        """
        Tests that debugger and target IDs can be passed through startDebugging
        for debugger reuse. This verifies the infrastructure for child DAP
        sessions to reuse the parent's debugger and attach to an existing target.
        """
        program = self.getBuildArtifact("a.out")
        source = "main.c"
        self.build_and_launch(program)

        breakpoint_line = line_number(source, "// breakpoint")
        self.set_source_breakpoints(source, [breakpoint_line])
        self.continue_to_next_stop()

        # Use mock IDs to test the infrastructure
        # In a real scenario, these would come from the parent session
        debugger_id = 1
        target_id = 100

        # Send a startDebugging request with debuggerId and targetId
        # This simulates creating a child DAP session that reuses the debugger
        session = {"session": {"debuggerId": debugger_id, "targetId": target_id}}
        self.dap_server.request_evaluate(
            f"`lldb-dap start-debugging attach '{json.dumps(session)}'",
            context="repl",
        )

        self.continue_to_exit()

        # Verify the reverse request was sent with the correct IDs
        self.assertEqual(
            len(self.dap_server.reverse_requests),
            1,
            "Should have received one startDebugging reverse request",
        )

        request = self.dap_server.reverse_requests[0]
        self.assertEqual(request["command"], "startDebugging")
        self.assertEqual(request["arguments"]["request"], "attach")

        session = request["arguments"]["configuration"]["session"]
        self.assertEqual(
            session["debuggerId"],
            debugger_id,
            "Reverse request should include debugger ID",
        )
        self.assertEqual(
            session["targetId"],
            target_id,
            "Reverse request should include target ID",
        )



"""Test calling functions in class methods."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestObjCClassMethod(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers to break inside main().
        self.main_source = "test.m"
        self.break_line = line_number(self.main_source, "// Set breakpoint here.")

    @add_test_categories(["pyapi"])
    def test_with_python_api(self):
        """Test calling functions in class methods."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        bpt = target.BreakpointCreateByLocation(self.main_source, self.break_line)
        self.assertTrue(bpt, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        thread_list = lldbutil.get_threads_stopped_at_breakpoint(process, bpt)

        # Make sure we stopped at the first breakpoint.
        self.assertNotEqual(len(thread_list), 0, "No thread stopped at our breakpoint.")
        self.assertEqual(
            len(thread_list), 1, "More than one thread stopped at our breakpoint."
        )

        frame = thread_list[0].GetFrameAtIndex(0)
        self.assertTrue(frame, "Got a valid frame 0 frame.")

        # Now make sure we can call a method that returns a struct without
        # crashing.
        cmd_value = frame.EvaluateExpression("[provider getRange]")
        self.assertTrue(cmd_value.IsValid())



"""Test that lldb will read additional registers from Mach-O LC_NOTE metadata."""

import os
import re
import subprocess

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestMetadataRegisters(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessDarwin
    @skipIfRemote
    def test_add_registers_via_metadata(self):
        self.build()
        self.aout_exe = self.getBuildArtifact("a.out")
        lldb_corefile = self.getBuildArtifact("lldb.core")
        metadata_corefile = self.getBuildArtifact("metadata.core")
        bad_metadata1_corefile = self.getBuildArtifact("bad-metadata1.core")
        bad_metadata2_corefile = self.getBuildArtifact("bad-metadata2.core")
        add_lcnote = self.getBuildArtifact("add-lcnote")

        (target, process, t, bp) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )

        self.assertTrue(process.IsValid())

        if self.TraceOn():
            self.runCmd("bt")
            self.runCmd("reg read -a")

        self.runCmd("process save-core " + lldb_corefile)
        process.Kill()
        target.Clear()

        cmd = (
            add_lcnote
            + " -r"
            + " -i '%s'" % lldb_corefile
            + " -o '%s'" % metadata_corefile
            + " -n 'process metadata' "
            + " -s '"
            + """{"threads":[{"register_info":
                   {"sets":["Special Registers", "General Purpose Registers"],
                    "registers":[
                       {"name":"jar", "value":10, "bitsize": 32, "set": 0},
                       {"name":"bar", "value":65537, "bitsize":16, "set":0},
                       {"name":"mar", "value":65537, "bitsize":32, "set":0},
                       {"name":"anotherpc", "value":55, "bitsize":64, "set": 1}]}}]}"""
            + "'"
        )
        call(cmd, shell=True)

        cmd = (
            add_lcnote
            + " -r"
            + " -i '%s'" % lldb_corefile
            + " -o '%s'" % bad_metadata1_corefile
            + " -n 'process metadata' "
            + " -s '"
            + """{lol im bad json}"""
            + "'"
        )
        call(cmd, shell=True)

        cmd = (
            add_lcnote
            + " -r"
            + " -i '%s'" % lldb_corefile
            + " -o '%s'" % bad_metadata2_corefile
            + " -n 'process metadata' "
            + " -s '"
            + """{"threads":[
                   {"register_info":
                     {"sets":["a"],"registers":[{"name":"a", "value":1, "bitsize": 32, "set": 0}]}},
                   {"register_info":
                     {"sets":["a"],"registers":[{"name":"a", "value":1, "bitsize": 32, "set": 0}]}}
                 ]}"""
            + "'"
        )
        call(cmd, shell=True)

        # Now load the corefile
        target = self.dbg.CreateTarget("")
        process = target.LoadCore(metadata_corefile)
        self.assertTrue(process.IsValid())
        if self.TraceOn():
            self.runCmd("bt")
            self.runCmd("reg read -a")

        thread = process.GetSelectedThread()
        frame = thread.GetFrameAtIndex(0)

        # Register sets will be
        #    from LC_THREAD:
        #       General Purpose Registers
        #       Floating Point Registers
        #       Exception State Registers
        #    from LC_NOTE metadata:
        #       Special Registers
        self.assertEqual(frame.registers[0].GetName(), "General Purpose Registers")
        self.assertEqual(frame.registers[3].GetName(), "Special Registers")

        anotherpc = frame.registers[0].GetChildMemberWithName("anotherpc")
        self.assertTrue(anotherpc.IsValid())
        self.assertEqual(anotherpc.GetValueAsUnsigned(), 0x37)

        jar = frame.registers[3].GetChildMemberWithName("jar")
        self.assertTrue(jar.IsValid())
        self.assertEqual(jar.GetValueAsUnsigned(), 10)
        self.assertEqual(jar.GetByteSize(), 4)

        bar = frame.registers[3].GetChildMemberWithName("bar")
        self.assertTrue(bar.IsValid())
        self.assertEqual(bar.GetByteSize(), 2)

        mar = frame.registers[3].GetChildMemberWithName("mar")
        self.assertTrue(mar.IsValid())
        self.assertEqual(mar.GetValueAsUnsigned(), 0x10001)
        self.assertEqual(mar.GetByteSize(), 4)

        # bad1 has invalid JSON, no additional registers
        target = self.dbg.CreateTarget("")
        process = target.LoadCore(bad_metadata1_corefile)
        self.assertTrue(process.IsValid())
        thread = process.GetSelectedThread()
        frame = thread.GetFrameAtIndex(0)
        gpr_found = False
        for regset in frame.registers:
            if regset.GetName() == "General Purpose Registers":
                gpr_found = True
        self.assertTrue(gpr_found)

        # bad2 has invalid JSON, more process metadata threads than
        # LC_THREADs, and should be rejected.
        target = self.dbg.CreateTarget("")
        process = target.LoadCore(bad_metadata2_corefile)
        self.assertTrue(process.IsValid())
        thread = process.GetSelectedThread()
        frame = thread.GetFrameAtIndex(0)
        metadata_regset_found = False
        for regset in frame.registers:
            if regset.GetName() == "a" or regset.GetName() == "b":
                metadata_regset_found = True
        self.assertFalse(metadata_regset_found)



"""Test that lldb can report the exception reason for threads in a corefile."""

import os
import re
import subprocess

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCorefileExceptionReason(TestBase):
    @no_debug_info_test
    @skipUnlessDarwin
    @skipIf(archs=no_match(["arm64", "arm64e"]))
    @skipIfRemote
    def test(self):
        corefile = self.getBuildArtifact("process.core")
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("continue")

        self.runCmd("process save-core -s stack " + corefile)
        live_tids = []
        if self.TraceOn():
            self.runCmd("thread list")
        for t in process.threads:
            live_tids.append(t.GetThreadID())
        process.Kill()
        self.dbg.DeleteTarget(target)

        # Now load the corefile
        target = self.dbg.CreateTarget("")
        process = target.LoadCore(corefile)
        thread = process.GetSelectedThread()
        self.assertTrue(process.GetSelectedThread().IsValid())
        if self.TraceOn():
            self.runCmd("image list")
            self.runCmd("bt")
            self.runCmd("fr v")

        self.assertEqual(
            thread.stop_description, "ESR_EC_DABORT_EL0 (fault address: 0x0)"
        )

        if self.TraceOn():
            self.runCmd("thread list")
        for i in range(process.GetNumThreads()):
            t = process.GetThreadAtIndex(i)
            self.assertEqual(t.GetThreadID(), live_tids[i])



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import transform
from mlir.dialects.transform import sparse_tensor


def run(f):
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            sequence = transform.SequenceOp(
                transform.FailurePropagationMode.Propagate,
                [],
                transform.AnyOpType.get(),
            )
            with InsertionPoint(sequence.body):
                f(sequence.bodyTarget)
                transform.YieldOp()
        print("\nTEST:", f.__name__)
        print(module)
    return f


@run
def testMatchSparseInOut(target):
    sparse_tensor.MatchSparseInOut(transform.AnyOpType.get(), target)
    # CHECK-LABEL: TEST: testMatchSparseInOut
    # CHECK:       transform.sequence
    # CHECK-NEXT:  ^{{.*}}(%[[ARG0:.*]]: !transform.any_op):
    # CHECK-NEXT:    transform.sparse_tensor.match.sparse_inout %[[ARG0]]



"""
Test SBCompileUnit APIs.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CompileUnitAPITestCase(TestBase):
    def test(self):
        """Exercise some SBCompileUnit APIs."""
        self.build()

        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )
        self.assertTrue(target, VALID_TARGET)
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertTrue(bkpt and bkpt.GetNumLocations() == 1, VALID_BREAKPOINT)

        self.assertTrue(
            thread.IsValid(),
            "There should be a thread stopped due to breakpoint condition",
        )
        frame0 = thread.GetFrameAtIndex(0)
        line_entry = frame0.GetLineEntry()

        sc_list = target.FindCompileUnits(line_entry.GetFileSpec())
        self.assertGreater(sc_list.GetSize(), 0)

        main_cu = sc_list.compile_units[0]
        self.assertTrue(main_cu.IsValid(), "Main executable CU is not valid")

        self.assertEqual(
            main_cu.FindLineEntryIndex(line_entry, True),
            main_cu.FindLineEntryIndex(
                0, line_entry.GetLine(), line_entry.GetFileSpec(), True
            ),
        )



"""
Tests that C++ expression evaluation can
disambiguate between rvalue and lvalue
reference-qualified functions.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestFunctionRefQualifiers(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "Break here", lldb.SBFileSpec("main.cpp")
        )

        # const lvalue
        self.expect_expr("const_foo.func()", result_type="uint32_t", result_value="0")

        # const rvalue
        self.expect_expr(
            "static_cast<Foo const&&>(Foo{}).func()",
            result_type="int64_t",
            result_value="1",
        )

        # non-const lvalue
        self.expect_expr("foo.func()", result_type="uint32_t", result_value="2")

        # non-const rvalue
        self.expect_expr("Foo{}.func()", result_type="int64_t", result_value="3")

        self.filecheck("target modules dump ast", __file__)
        # CHECK:      |-CXXMethodDecl {{.*}} func 'uint32_t () const &'
        # CHECK-NEXT: | `-AsmLabelAttr {{.*}}
        # CHECK-NEXT: |-CXXMethodDecl {{.*}} func 'int64_t () const &&'
        # CHECK-NEXT: | `-AsmLabelAttr {{.*}}
        # CHECK-NEXT: |-CXXMethodDecl {{.*}} func 'uint32_t () &'
        # CHECK-NEXT: | `-AsmLabelAttr {{.*}}
        # CHECK-NEXT: `-CXXMethodDecl {{.*}} func 'int64_t () &&'
        # CHECK-NEXT:   `-AsmLabelAttr {{.*}}



# -*- Python -*-

import shlex
import lit.util

from lit.llvm import llvm_config
from lit.llvm.subst import ToolSubst, FindTool


def shjoin(args, sep=" "):
    return sep.join([shlex.quote(arg) for arg in args])


# Configuration file for the 'lit' test runner.

# name: The name of this test suite.
config.name = "flang-rt"

# TODO: Consolidate the logic for turning on the internal shell by default for all LLVM test suites.
# See https://github.com/llvm/llvm-project/issues/106636 for more details.
#
# We prefer the lit internal shell which provides a better user experience on failures
# unless the user explicitly disables it with LIT_USE_INTERNAL_SHELL=0 env var.
use_lit_shell = True
lit_shell_env = os.environ.get("LIT_USE_INTERNAL_SHELL")
if lit_shell_env:
    use_lit_shell = lit.util.pythonize_bool(lit_shell_env)

# testFormat: The test format to use to interpret tests.
#
# For now we require '&&' between commands, until they get globally killed and
# the test runner updated.
config.test_format = lit.formats.ShTest(not use_lit_shell)

# suffixes: A list of file extensions to treat as test files.
config.suffixes = [
    ".c",
    ".cpp",
    ".f",
    ".F",
    ".ff",
    ".FOR",
    ".for",
    ".f77",
    ".f90",
    ".F90",
    ".ff90",
    ".f95",
    ".F95",
    ".ff95",
    ".fpp",
    ".FPP",
    ".cuf",
    ".CUF",
    ".f18",
    ".F18",
    ".f03",
    ".F03",
    ".f08",
    ".F08",
    ".ll",
    ".fir",
    ".mlir",
]

llvm_config.use_default_substitutions()

# test_source_root: The root path where tests are located.
config.test_source_root = os.path.dirname(__file__)

# test_exec_root: The root path where tests should be run.
# lit writes a '.lit_test_times.txt' file into this directory.
config.test_exec_root = config.flang_rt_binary_test_dir

# On MacOS, some tests need -isysroot to build binaries.
isysroot_flag = []
if config.osx_sysroot:
    isysroot_flag = ["-isysroot", config.osx_sysroot]
config.substitutions.append(("%isysroot", " ".join(isysroot_flag)))

tools = [
    ToolSubst(
        "%flang",
        command=config.flang,
        unresolved="fatal",
    ),
    ToolSubst(
        "%clang",
        command=FindTool("clang"),
        unresolved="fatal",
    ),
    ToolSubst("%cc", command=config.cc, unresolved="fatal"),
]
llvm_config.add_tool_substitutions(tools)

# Let tests find LLVM's standard tools (FileCheck, split-file, not, ...)
llvm_config.with_environment("PATH", config.llvm_tools_dir, append_path=True)

# Include path for C headers that define Flang's Fortran ABI.
config.substitutions.append(
    ("%include", os.path.join(config.flang_source_dir, "include"))
)

# Library path of libflang_rt.runtime.a/.so (for lib search path when using non-Flang driver for linking and LD_LIBRARY_PATH)
config.substitutions.append(("%libdir", config.flang_rt_output_resource_lib_dir))

# For CUDA offloading, additional steps (device linking) and libraries (cudart) are needed.
if config.flang_rt_experimental_offload_support == "CUDA":
    config.available_features.add("offload-cuda")



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
import mlir.dialects.spirv as spirv


def run(f):
    print("\nTEST:", f.__name__)
    f()


# CHECK-LABEL: TEST: testConstantOp
@run
def testConstantOps():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            i32 = IntegerType.get_signless(32)
            spirv.ConstantOp(value=IntegerAttr.get(i32, 42), constant=i32)
        # CHECK: spirv.Constant 42 : i32
        print(module)



"""
Frame provider that reproduces the circular dependency issue.

This provider accesses input_frames and calls methods on them,
which before the fix would cause a circular dependency.
"""

import lldb
from lldb.plugins.scripted_process import ScriptedFrame
from lldb.plugins.scripted_frame_provider import ScriptedFrameProvider


class CustomScriptedFrame(ScriptedFrame):
    """Custom scripted frame with full control over frame behavior."""

    def __init__(self, thread, idx, pc, function_name):
        args = lldb.SBStructuredData()
        super().__init__(thread, args)

        self.idx = idx
        self.pc = pc
        self.function_name = function_name

    def get_id(self):
        """Return the frame index."""
        return self.idx

    def get_pc(self):
        """Return the program counter."""
        return self.pc

    def get_function_name(self):
        """Return the function name."""
        return self.function_name

    def is_artificial(self):
        """Mark as artificial frame."""
        return False

    def is_hidden(self):
        """Not hidden."""
        return False

    def get_register_context(self):
        return None


class ScriptedFrameObjectProvider(ScriptedFrameProvider):
    """
    Provider that returns ScriptedFrame objects and accesses input_frames.

    This provider demonstrates the circular dependency bug fix:
    1. During get_frame_at_index(), we access input_frames[idx]
    2. We call frame.GetFunctionName() and frame.GetPC() on input frames
    3. Before the fix: These calls would trigger ExecutionContextRef::GetFrameSP()
       which would call Thread::GetStackFrameList() -> circular dependency!
    4. After the fix: ExecutionContextRef uses the remembered frame list -> no circular dependency
    """

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)
        self.replacement_count = 0
        if self.target.process:
            baz_symbol_ctx = self.target.FindFunctions("baz")
            self.baz_symbol_ctx = None
            if len(baz_symbol_ctx) == 1:
                self.baz_symbol_ctx = baz_symbol_ctx[0]

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Provider that replaces 'bar' function with 'baz'"

    def get_frame_at_index(self, idx):
        """
        Replace frames named 'bar' with custom frames named 'baz'.

        This accesses input_frames and calls methods on them, which would
        trigger the circular dependency bug before the fix.
        """
        if idx < len(self.input_frames):
            # This access and method calls would cause circular dependency before fix!
            frame = self.input_frames[idx]

            # Calling GetFunctionName() triggers ExecutionContextRef resolution.
            function_name = frame.GetFunctionName()

            if function_name == "bar" and self.baz_symbol_ctx:
                # Replace "bar" with "baz".
                baz_func = self.baz_symbol_ctx.GetFunction()
                new_function_name = baz_func.GetName()
                pc = baz_func.GetStartAddress().GetLoadAddress(self.target)
                custom_frame = CustomScriptedFrame(
                    self.thread, idx, pc, new_function_name
                )
                self.replacement_count += 1
                return custom_frame

            # Pass through other frames by returning their index.
            return idx

        return None


class HandleCommandInInitProvider(ScriptedFrameProvider):
    """
    Provider that calls HandleCommand during __init__.

    This reproduces the circular dependency by running a command that
    accesses the thread's stack frames while the provider is being
    initialized (i.e. while the SyntheticStackFrameList is being built).
    Before the fix, HandleCommand("bt") would call Thread::GetStackFrameList()
    which would try to create the SyntheticStackFrameList again -> deadlock.
    """

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)
        # Running "bt" during init triggers frame access on the thread,
        # which before the fix would cause a circular dependency.
        result = lldb.SBCommandReturnObject()
        self.thread.GetProcess().GetTarget().GetDebugger().GetCommandInterpreter().HandleCommand(
            "bt", result
        )
        self.init_succeeded = result.Succeeded()

    @staticmethod
    def get_description():
        return "Provider that calls HandleCommand('bt') in __init__"

    def get_frame_at_index(self, idx):
        if idx < len(self.input_frames):
            return idx
        return None


class EvaluateExpressionInGetFrameProvider(ScriptedFrameProvider):
    """
    Provider that calls EvaluateExpression in get_frame_at_index.

    This reproduces the circular dependency by evaluating an expression
    that accesses the thread's stack frames while the provider is fetching
    frames. Before the fix, EvaluateExpression would call
    Thread::GetStackFrameList() which would re-enter the
    SyntheticStackFrameList -> circular dependency.
    """

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        return "Provider that calls EvaluateExpression in get_frame_at_index"

    def get_frame_at_index(self, idx):
        if idx < len(self.input_frames):
            frame = self.input_frames[idx]
            # Evaluating an expression that calls a function triggers frame
            # access on the thread, which before the fix would cause a
            # circular dependency.
            frame.EvaluateExpression("baz()")
            return idx
        return None
